#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.constants import WORKDAY
from holidays.countries.philippines import Philippines
from tests.common import CommonCountryTests


class TestPhilippines(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Philippines)

    def test_no_holidays(self):
        super().test_no_holidays()

        self.assertNoHolidays(Philippines(categories=WORKDAY, years=range(self.start_year, 2009)))

    def test_special_holidays(self):
        self.assertHoliday(
            # Additional special (non-working) day.
            "2008-12-26",
            "2008-12-29",
            "2009-11-02",
            "2009-12-24",
            "2010-12-24",
            "2012-11-02",
            "2013-11-02",
            "2013-12-24",
            "2014-12-24",
            "2014-12-26",
            "2015-01-02",
            "2015-12-24",
            "2016-01-02",
            "2016-10-31",
            "2016-12-24",
            "2017-01-02",
            "2017-10-31",
            "2018-05-14",
            "2018-11-02",
            "2018-12-24",
            "2019-05-13",
            "2019-11-02",
            "2019-12-24",
            "2020-11-02",
            "2020-12-24",
            "2022-05-09",
            "2022-10-31",
            "2023-01-02",
            "2023-10-30",
            "2023-11-02",
            "2023-12-26",
            "2024-02-09",
            "2024-11-02",
            "2024-12-24",
            "2025-05-12",
            "2025-07-27",
            "2025-10-31",
            "2025-12-24",
            "2026-11-02",
            "2026-12-24",
        )

    def test_new_years_day(self):
        self.assertHolidayName("New Year's Day", (f"{year}-01-01" for year in self.full_range))

    def test_chinese_new_year(self):
        name = "Chinese New Year"
        self.assertHolidayName(
            name,
            "2012-01-23",
            "2013-02-10",
            "2014-01-31",
            "2015-02-19",
            "2016-02-08",
            "2017-01-28",
            "2018-02-16",
            "2019-02-05",
            "2020-01-25",
            "2021-02-12",
            "2022-02-01",
            "2024-02-10",
            "2025-01-29",
            "2026-02-17",
        )
        self.assertHolidayName(name, range(2012, 2023), range(2024, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 2012), 2023)

    def test_edsa_people_power_revolution_anniversary(self):
        name = "EDSA People Power Revolution Anniversary"
        self.assertHolidayName(
            name,
            (f"{year}-02-25" for year in range(2018, 2023)),
            "2016-02-25",
            "2023-02-24",
        )
        self.assertNoHolidayName(
            name, range(self.start_year, 2016), 2017, range(2024, self.end_year)
        )
        self.assertWorkdayHolidayName(
            name, (f"{year}-02-25" for year in range(2025, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 2025))

    def test_maundy_thursday(self):
        name = "Maundy Thursday"
        self.assertHolidayName(
            name,
            "2020-04-09",
            "2021-04-01",
            "2022-04-14",
            "2023-04-06",
            "2024-03-28",
            "2025-04-17",
        )
        self.assertHolidayName(name, self.full_range)

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_black_saturday(self):
        name = "Black Saturday"
        self.assertHolidayName(
            name,
            "2020-04-11",
            "2021-04-03",
            "2022-04-16",
            "2023-04-08",
            "2024-03-30",
            "2025-04-19",
        )
        self.assertHolidayName(name, range(2013, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 2013))

    def test_day_of_valor(self):
        self.assertHolidayName(
            "Araw ng Kagitingan",
            (f"{year}-04-09" for year in set(self.full_range) - {2008, 2009, 2023}),
            "2008-04-07",
            "2009-04-06",
            "2023-04-10",
        )

    def test_labor_day(self):
        self.assertHolidayName("Labor Day", (f"{year}-05-01" for year in self.full_range))

    def test_independence_day(self):
        self.assertHolidayName(
            "Independence Day",
            (
                f"{year}-06-12"
                for year in (*range(self.start_year, 2007), *range(2011, self.end_year))
            ),
            "2007-06-11",
            "2008-06-09",
            "2009-06-12",
            "2010-06-14",
        )

    def test_iglesia_ni_cristo(self):
        # 2025 special non-working day is marked as an additional entry for now,
        # not part of this test case.
        name = "Founding Anniversary of Iglesia ni Cristo"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(
            name,
            (f"{year}-07-27" for year in (*range(2009, 2025), *range(2026, self.end_year))),
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 2009), 2025)

    def test_ninoy_aquino_day(self):
        name = "Ninoy Aquino Day"
        self.assertHolidayName(
            name,
            (
                f"{year}-08-21"
                for year in set(range(2004, self.end_year)) - {2007, 2008, 2010, 2024}
            ),
            "2007-08-20",
            "2008-08-18",
            "2010-08-23",
            "2024-08-23",
        )
        self.assertNoHolidayName(name, range(self.start_year, 2004))

    def test_national_heroes_day(self):
        name = "National Heroes Day"
        self.assertHolidayName(
            name,
            "2004-08-29",
            "2005-08-28",
            "2006-08-27",
            "2007-08-27",
            "2008-08-25",
            "2016-08-29",
            "2017-08-28",
            "2018-08-27",
            "2019-08-26",
            "2020-08-31",
            "2021-08-30",
            "2022-08-29",
            "2023-08-28",
            "2024-08-26",
        )
        self.assertHolidayName(name, self.full_range)

    def test_all_saints_day(self):
        self.assertHolidayName("All Saints' Day", (f"{year}-11-01" for year in self.full_range))

    def test_bonifacio_day(self):
        self.assertHolidayName(
            "Bonifacio Day",
            (
                f"{year}-11-30"
                for year in (
                    *range(self.start_year, 2008),
                    *range(2011, 2023),
                    *range(2024, self.end_year),
                )
            ),
            "2008-12-01",
            "2009-11-30",
            "2010-11-29",
            "2023-11-27",
        )

    def test_immaculate_conception_day(self):
        name = "Feast of the Immaculate Conception of Mary"
        self.assertHolidayName(name, (f"{year}-12-08" for year in range(2019, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2019))

    def test_christmas_day(self):
        self.assertHolidayName("Christmas Day", (f"{year}-12-25" for year in self.full_range))

    def test_rizal_day(self):
        self.assertHolidayName(
            "Rizal Day",
            (
                f"{year}-12-30"
                for year in (*range(self.start_year, 2010), *range(2011, self.end_year))
            ),
            "2010-12-27",
        )

    def test_last_day_of_year(self):
        name = "Last Day of the Year"
        self.assertHolidayName(
            name,
            (
                f"{year}-12-31"
                for year in (*range(self.start_year, 2021), *range(2023, self.end_year))
            ),
        )
        self.assertNoHolidayName(name, 2021, 2022)

    def test_eid_al_fitr(self):
        name = "Eid'l Fitr"
        self.assertHolidayName(
            name,
            "2020-05-25",
            "2021-05-13",
            "2022-05-03",
            "2023-04-21",
            "2024-04-10",
            "2025-04-01",
        )
        self.assertIslamicNoEstimatedHolidayName(name, range(2002, self.end_year))
        self.assertNoIslamicNoEstimatedHolidayName(name, range(self.start_year, 2002))

    def test_eid_al_adha(self):
        name = "Eid'l Adha"
        self.assertHolidayName(
            name,
            "2020-07-31",
            "2021-07-20",
            "2022-07-09",
            "2023-06-28",
            "2024-06-17",
            "2025-06-06",
        )
        self.assertIslamicNoEstimatedHolidayName(name, range(2010, self.end_year))
        self.assertNoIslamicNoEstimatedHolidayName(name, range(self.start_year, 2010))

    def test_2018(self):
        self.assertHolidaysInYear(
            2018,
            ("2018-01-01", "New Year's Day"),
            ("2018-02-16", "Chinese New Year"),
            ("2018-02-25", "EDSA People Power Revolution Anniversary"),
            ("2018-03-29", "Maundy Thursday"),
            ("2018-03-30", "Good Friday"),
            ("2018-03-31", "Black Saturday"),
            ("2018-04-09", "Araw ng Kagitingan"),
            ("2018-05-01", "Labor Day"),
            ("2018-05-14", "Elections special (non-working) day"),
            ("2018-06-12", "Independence Day"),
            ("2018-06-15", "Eid'l Fitr"),
            ("2018-08-21", "Eid'l Adha; Ninoy Aquino Day"),
            ("2018-08-27", "National Heroes Day"),
            ("2018-11-01", "All Saints' Day"),
            ("2018-11-02", "Additional special (non-working) day"),
            ("2018-11-30", "Bonifacio Day"),
            ("2018-12-24", "Additional special (non-working) day"),
            ("2018-12-25", "Christmas Day"),
            ("2018-12-30", "Rizal Day"),
            ("2018-12-31", "Last Day of the Year"),
        )

    def test_2019(self):
        self.assertHolidaysInYear(
            2019,
            ("2019-01-01", "New Year's Day"),
            ("2019-02-05", "Chinese New Year"),
            ("2019-02-25", "EDSA People Power Revolution Anniversary"),
            ("2019-04-09", "Araw ng Kagitingan"),
            ("2019-04-18", "Maundy Thursday"),
            ("2019-04-19", "Good Friday"),
            ("2019-04-20", "Black Saturday"),
            ("2019-05-01", "Labor Day"),
            ("2019-05-13", "Elections special (non-working) day"),
            ("2019-06-05", "Eid'l Fitr"),
            ("2019-06-12", "Independence Day"),
            ("2019-08-12", "Eid'l Adha"),
            ("2019-08-21", "Ninoy Aquino Day"),
            ("2019-08-26", "National Heroes Day"),
            ("2019-11-01", "All Saints' Day"),
            ("2019-11-02", "Additional special (non-working) day"),
            ("2019-11-30", "Bonifacio Day"),
            ("2019-12-08", "Feast of the Immaculate Conception of Mary"),
            ("2019-12-24", "Additional special (non-working) day"),
            ("2019-12-25", "Christmas Day"),
            ("2019-12-30", "Rizal Day"),
            ("2019-12-31", "Last Day of the Year"),
        )

    def test_2020(self):
        self.assertHolidaysInYear(
            2020,
            ("2020-01-01", "New Year's Day"),
            ("2020-01-25", "Chinese New Year"),
            ("2020-02-25", "EDSA People Power Revolution Anniversary"),
            ("2020-04-09", "Araw ng Kagitingan; Maundy Thursday"),
            ("2020-04-10", "Good Friday"),
            ("2020-04-11", "Black Saturday"),
            ("2020-05-01", "Labor Day"),
            ("2020-05-25", "Eid'l Fitr"),
            ("2020-06-12", "Independence Day"),
            ("2020-07-31", "Eid'l Adha"),
            ("2020-08-21", "Ninoy Aquino Day"),
            ("2020-08-31", "National Heroes Day"),
            ("2020-11-01", "All Saints' Day"),
            ("2020-11-02", "Additional special (non-working) day"),
            ("2020-11-30", "Bonifacio Day"),
            ("2020-12-08", "Feast of the Immaculate Conception of Mary"),
            ("2020-12-24", "Additional special (non-working) day"),
            ("2020-12-25", "Christmas Day"),
            ("2020-12-30", "Rizal Day"),
            ("2020-12-31", "Last Day of the Year"),
        )

    def test_2021(self):
        self.assertHolidaysInYear(
            2021,
            ("2021-01-01", "New Year's Day"),
            ("2021-02-12", "Chinese New Year"),
            ("2021-02-25", "EDSA People Power Revolution Anniversary"),
            ("2021-04-01", "Maundy Thursday"),
            ("2021-04-02", "Good Friday"),
            ("2021-04-03", "Black Saturday"),
            ("2021-04-09", "Araw ng Kagitingan"),
            ("2021-05-01", "Labor Day"),
            ("2021-05-13", "Eid'l Fitr"),
            ("2021-06-12", "Independence Day"),
            ("2021-07-20", "Eid'l Adha"),
            ("2021-08-21", "Ninoy Aquino Day"),
            ("2021-08-30", "National Heroes Day"),
            ("2021-11-01", "All Saints' Day"),
            ("2021-11-30", "Bonifacio Day"),
            ("2021-12-08", "Feast of the Immaculate Conception of Mary"),
            ("2021-12-25", "Christmas Day"),
            ("2021-12-30", "Rizal Day"),
        )

    def test_2022(self):
        self.assertHolidaysInYear(
            2022,
            ("2022-01-01", "New Year's Day"),
            ("2022-02-01", "Chinese New Year"),
            ("2022-02-25", "EDSA People Power Revolution Anniversary"),
            ("2022-04-09", "Araw ng Kagitingan"),
            ("2022-04-14", "Maundy Thursday"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-16", "Black Saturday"),
            ("2022-05-01", "Labor Day"),
            ("2022-05-03", "Eid'l Fitr"),
            ("2022-05-09", "Elections special (non-working) day"),
            ("2022-06-12", "Independence Day"),
            ("2022-07-09", "Eid'l Adha"),
            ("2022-08-21", "Ninoy Aquino Day"),
            ("2022-08-29", "National Heroes Day"),
            ("2022-10-31", "Additional special (non-working) day"),
            ("2022-11-01", "All Saints' Day"),
            ("2022-11-30", "Bonifacio Day"),
            ("2022-12-08", "Feast of the Immaculate Conception of Mary"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-30", "Rizal Day"),
        )

    def test_2023(self):
        self.assertHolidaysInYear(
            2023,
            ("2023-01-01", "New Year's Day"),
            ("2023-01-02", "Additional special (non-working) day"),
            ("2023-02-24", "EDSA People Power Revolution Anniversary"),
            ("2023-04-06", "Maundy Thursday"),
            ("2023-04-07", "Good Friday"),
            ("2023-04-08", "Black Saturday"),
            ("2023-04-10", "Araw ng Kagitingan"),
            ("2023-04-21", "Eid'l Fitr"),
            ("2023-05-01", "Labor Day"),
            ("2023-06-12", "Independence Day"),
            ("2023-06-28", "Eid'l Adha"),
            ("2023-08-21", "Ninoy Aquino Day"),
            ("2023-08-28", "National Heroes Day"),
            ("2023-10-30", "Elections special (non-working) day"),
            ("2023-11-01", "All Saints' Day"),
            ("2023-11-02", "Additional special (non-working) day"),
            ("2023-11-27", "Bonifacio Day"),
            ("2023-12-08", "Feast of the Immaculate Conception of Mary"),
            ("2023-12-25", "Christmas Day"),
            ("2023-12-26", "Additional special (non-working) day"),
            ("2023-12-30", "Rizal Day"),
            ("2023-12-31", "Last Day of the Year"),
        )

    def test_2025(self):
        self.assertHolidaysInYear(
            2025,
            ("2025-01-01", "New Year's Day"),
            ("2025-01-29", "Chinese New Year"),
            ("2025-04-01", "Eid'l Fitr"),
            ("2025-04-09", "Araw ng Kagitingan"),
            ("2025-04-17", "Maundy Thursday"),
            ("2025-04-18", "Good Friday"),
            ("2025-04-19", "Black Saturday"),
            ("2025-05-01", "Labor Day"),
            ("2025-05-12", "Elections special (non-working) day"),
            ("2025-06-06", "Eid'l Adha"),
            ("2025-06-12", "Independence Day"),
            ("2025-07-27", "Additional special (non-working) day"),
            ("2025-08-21", "Ninoy Aquino Day"),
            ("2025-08-25", "National Heroes Day"),
            ("2025-10-31", "All Saints' Day Eve"),
            ("2025-11-01", "All Saints' Day"),
            ("2025-11-30", "Bonifacio Day"),
            ("2025-12-08", "Feast of the Immaculate Conception of Mary"),
            ("2025-12-24", "Christmas Eve"),
            ("2025-12-25", "Christmas Day"),
            ("2025-12-30", "Rizal Day"),
            ("2025-12-31", "Last Day of the Year"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2024-01-01", "New Year's Day"),
            ("2024-02-09", "Additional special (non-working) day"),
            ("2024-02-10", "Chinese New Year"),
            ("2024-03-28", "Maundy Thursday"),
            ("2024-03-29", "Good Friday"),
            ("2024-03-30", "Black Saturday"),
            ("2024-04-09", "Araw ng Kagitingan"),
            ("2024-04-10", "Eid'l Fitr"),
            ("2024-05-01", "Labor Day"),
            ("2024-06-12", "Independence Day"),
            ("2024-06-17", "Eid'l Adha"),
            ("2024-07-27", "Founding Anniversary of Iglesia ni Cristo"),
            ("2024-08-23", "Ninoy Aquino Day"),
            ("2024-08-26", "National Heroes Day"),
            ("2024-11-01", "All Saints' Day"),
            ("2024-11-02", "Additional special (non-working) day"),
            ("2024-11-30", "Bonifacio Day"),
            ("2024-12-08", "Feast of the Immaculate Conception of Mary"),
            ("2024-12-24", "Additional special (non-working) day"),
            ("2024-12-25", "Christmas Day"),
            ("2024-12-30", "Rizal Day"),
            ("2024-12-31", "Last Day of the Year"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2024-01-01", "New Year's Day"),
            ("2024-02-09", "Additional special (non-working) day"),
            ("2024-02-10", "Chinese New Year"),
            ("2024-03-28", "Maundy Thursday"),
            ("2024-03-29", "Good Friday"),
            ("2024-03-30", "Black Saturday"),
            ("2024-04-09", "Day of Valor"),
            ("2024-04-10", "Eid al-Fitr"),
            ("2024-05-01", "Labor Day"),
            ("2024-06-12", "Independence Day"),
            ("2024-06-17", "Eid al-Adha"),
            ("2024-07-27", "Founding Anniversary of Iglesia ni Cristo"),
            ("2024-08-23", "Ninoy Aquino Day"),
            ("2024-08-26", "National Heroes Day"),
            ("2024-11-01", "All Saints' Day"),
            ("2024-11-02", "Additional special (non-working) day"),
            ("2024-11-30", "Bonifacio Day"),
            ("2024-12-08", "Immaculate Conception"),
            ("2024-12-24", "Additional special (non-working) day"),
            ("2024-12-25", "Christmas Day"),
            ("2024-12-30", "Rizal Day"),
            ("2024-12-31", "New Year's Eve"),
        )

    def test_l10n_fil(self):
        self.assertLocalizedHolidays(
            "fil",
            ("2024-01-01", "Bagong Taon"),
            ("2024-02-09", "Karagdagang Espesyal na Araw (Walang Trabajo)"),
            ("2024-02-10", "Bagong Taon ng mga Tsino"),
            ("2024-03-28", "Huwebes Santo"),
            ("2024-03-29", "Biyernes Santo"),
            ("2024-03-30", "Sabado de Gloria"),
            ("2024-04-09", "Araw ng Kagitingan"),
            ("2024-04-10", "Eid al-Fitr"),
            ("2024-05-01", "Araw ng Paggawa"),
            ("2024-06-12", "Araw ng Kalayaan"),
            ("2024-06-17", "Eid al-Adha"),
            ("2024-07-27", "Anibersaryo ng Pagkatatag ng Iglesia ni Cristo"),
            ("2024-08-23", "Araw ng Kabayanihan ni Ninoy Aquino"),
            ("2024-08-26", "Araw ng mga Bayani"),
            ("2024-11-01", "Araw ng mga Santo"),
            ("2024-11-02", "Karagdagang Espesyal na Araw (Walang Trabajo)"),
            ("2024-11-30", "Araw ng Kabayanihan ni Bonifacio"),
            (
                "2024-12-08",
                "Dakilang Kapistahan ng Kalinis-linisang Paglilihi sa Mahal na Birheng Maria",
            ),
            ("2024-12-24", "Karagdagang Espesyal na Araw (Walang Trabajo)"),
            ("2024-12-25", "Pasko"),
            ("2024-12-30", "Araw ng Kabayanihan ni Rizal"),
            ("2024-12-31", "Bisperas ng Bagong Taon"),
        )

    def test_l10n_th(self):
        self.assertLocalizedHolidays(
            "th",
            ("2024-01-01", "วันขึ้นปีใหม่"),
            ("2024-02-09", "วันหยุดพิเศษ (เพิ่มเติม)"),
            ("2024-02-10", "วันตรุษจีน"),
            ("2024-03-28", "วันพฤหัสบดีศักดิ์สิทธิ์"),
            ("2024-03-29", "วันศุกร์ประเสริฐ"),
            ("2024-03-30", "วันเสาร์ศักดิ์สิทธิ์"),
            ("2024-04-09", "วันแห่งความกล้าหาญ"),
            ("2024-04-10", "วันอีฎิ้ลฟิตริ"),
            ("2024-05-01", "วันแรงงาน"),
            ("2024-06-12", "วันประกาศเอกราชสาธารณรัฐฟิลิปปินส์"),
            ("2024-06-17", "วันอีดิ้ลอัฎฮา"),
            ("2024-07-27", "วันครบรอบการสถาปนานิกายคริสตจักรของพระคริสต์"),
            ("2024-08-23", "วันนินอย อากีโน"),
            ("2024-08-26", "วันวีรบุรุษแห่งชาติ"),
            ("2024-11-01", "วันสมโภชนักบุญทั้งหลาย"),
            ("2024-11-02", "วันหยุดพิเศษ (เพิ่มเติม)"),
            ("2024-11-30", "วันโบนีฟาซีโอ"),
            ("2024-12-08", "วันสมโภชแม่พระผู้ปฏิสนธินิรมล"),
            ("2024-12-24", "วันหยุดพิเศษ (เพิ่มเติม)"),
            ("2024-12-25", "วันคริสต์มาส"),
            ("2024-12-30", "วันรีซัล"),
            ("2024-12-31", "วันสิ้นปี"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

import warnings
from unittest import TestCase

from holidays.countries.united_kingdom import UnitedKingdom
from tests.common import CommonCountryTests


class TestUnitedKingdom(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(UnitedKingdom)

    def setUp(self):
        super().setUp()
        warnings.simplefilter("ignore", category=DeprecationWarning)

    def test_subdiv_deprecation(self):
        self.assertDeprecatedSubdivisions("This subdivision is deprecated and will be removed")

    def test_no_holidays(self):
        super().test_no_holidays()

        for subdiv in UnitedKingdom.subdivisions:
            self.assertNoHolidays(UnitedKingdom(years=self.start_year - 1, subdiv=subdiv))

    def test_special_holidays(self):
        self.assertHoliday(
            "1977-06-07",
            "1981-07-29",
            "1999-12-31",
            "2002-06-03",
            "2011-04-29",
            "2012-06-05",
            "2022-06-03",
            "2022-09-19",
            "2023-05-08",
        )

    def test_special_subdiv_holidays(self):
        dts = ("2026-06-15",)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "SCT":
                self.assertSubdivSctHoliday(dts)
            else:
                self.assertNoHoliday(holidays, dts)

        self.assertNoHoliday(dts)

    def test_new_years_day(self):
        name = "New Year's Day"
        self.assertHolidayName(name, (f"{year}-01-01" for year in range(1975, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1975))

        obs_dts = (
            "2000-01-03",
            "2005-01-03",
            "2006-01-02",
            "2011-01-03",
            "2012-01-02",
            "2017-01-02",
            "2022-01-03",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_new_years_day_scotland(self):
        name_new_year = "New Year's Day"
        name_new_year_holiday = "New Year Holiday"

        nyh_obs_dts = (
            "2000-01-04",
            "2005-01-04",
            "2006-01-03",
            "2010-01-04",
            "2011-01-04",
            "2012-01-03",
            "2016-01-04",
            "2017-01-03",
            "2021-01-04",
            "2022-01-04",
            "2023-01-03",
        )
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "SCT":
                self.assertSubdivSctHolidayName(
                    name_new_year, (f"{year}-01-01" for year in self.full_range)
                )
                self.assertSubdivSctHolidayName(
                    f"{name_new_year} (observed)",
                    "2000-01-03",
                    "2005-01-03",
                    "2006-01-02",
                    "2011-01-03",
                    "2012-01-02",
                    "2017-01-02",
                    "2022-01-03",
                    "2023-01-02",
                )
                self.assertSubdivSctHolidayName(
                    name_new_year_holiday, (f"{year}-01-02" for year in self.full_range)
                )
                self.assertSubdivSctHolidayName(f"{name_new_year_holiday} (observed)", nyh_obs_dts)
                self.assertNoSubdivSctNonObservedHoliday(nyh_obs_dts)
            else:
                self.assertNoHolidayName(name_new_year_holiday, holidays)

    def test_saint_patricks_day(self):
        name = "Saint Patrick's Day"
        self.assertNoHolidayName(name)

        obs_dts = (
            "2001-03-19",
            "2002-03-18",
            "2007-03-19",
            "2012-03-19",
            "2013-03-18",
            "2018-03-19",
            "2019-03-18",
        )
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "NIR":
                self.assertSubdivNirHolidayName(
                    name, (f"{year}-03-17" for year in range(1903, self.end_year))
                )
                self.assertNoSubdivNirHolidayName(name, range(self.start_year, 1903))
                self.assertSubdivNirHolidayName(f"{name} (observed)", obs_dts)
                self.assertNoSubdivNirNonObservedHoliday(obs_dts)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Easter Monday"
        self.assertNoHolidayName(name)

        dts = (
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "SCT":
                self.assertNoSubdivSctHolidayName(name)
            else:
                self.assertHolidayName(name, holidays, dts)
                self.assertHolidayName(name, holidays, self.full_range)

    def test_may_day(self):
        name = "May Day"
        self.assertHolidayName(
            name,
            "1978-05-01",
            "1979-05-07",
            "1980-05-05",
            "1995-05-08",
            "1999-05-03",
            "2000-05-01",
            "2010-05-03",
            "2016-05-02",
            "2018-05-07",
            "2019-05-06",
            "2020-05-08",
        )
        self.assertHolidayName(name, range(1978, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 1978))

    def test_whit_monday(self):
        name = "Whit Monday"
        self.assertNoHolidayName(name)

        dts = (
            "1965-06-07",
            "1966-05-30",
            "1967-05-15",
            "1968-06-03",
            "1969-05-26",
            "1970-05-18",
        )
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "SCT":
                self.assertNoHolidayName(name, holidays)
            else:
                self.assertHolidayName(name, holidays, dts)
                self.assertHolidayName(name, holidays, range(self.start_year, 1971))
                self.assertNoHolidayName(name, holidays, range(1971, self.end_year))

    def test_spring_bank_holiday(self):
        name = "Spring Bank Holiday"
        self.assertHolidayName(
            name,
            "2002-06-04",
            "2012-06-04",
            "2020-05-25",
            "2021-05-31",
            "2022-06-02",
            "2023-05-29",
            "2024-05-27",
            "2025-05-26",
        )
        self.assertHolidayName(name, range(1971, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 1971))

    def test_battle_of_the_boyne_day(self):
        name = "Battle of the Boyne"
        self.assertNoHolidayName(name)

        obs_dts = (
            "2003-07-14",
            "2008-07-14",
            "2009-07-13",
            "2014-07-14",
            "2015-07-13",
            "2020-07-13",
        )
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "NIR":
                self.assertSubdivNirHolidayName(
                    name, (f"{year}-07-12" for year in self.full_range)
                )
                self.assertSubdivNirHolidayName(f"{name} (observed)", obs_dts)
                self.assertNoSubdivNirNonObservedHoliday(obs_dts)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_summer_bank_holiday(self):
        name = "Summer Bank Holiday"
        self.assertNoHolidayName(name)

        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "SCT":
                self.assertSubdivSctHolidayName(
                    name,
                    "2020-08-03",
                    "2021-08-02",
                    "2022-08-01",
                    "2023-08-07",
                    "2024-08-05",
                    "2025-08-04",
                )
                self.assertSubdivSctHolidayName(name, self.full_range)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_late_summer_bank_holiday(self):
        name = "Late Summer Bank Holiday"
        self.assertNoHolidayName(name)

        dts = (
            "2020-08-31",
            "2021-08-30",
            "2022-08-29",
            "2023-08-28",
            "2024-08-26",
            "2025-08-25",
        )
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "SCT":
                self.assertNoSubdivSctHolidayName(name)
            else:
                self.assertHolidayName(name, holidays, dts)
                self.assertHolidayName(name, holidays, range(1971, self.end_year))
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1971))

    def test_saint_andrews_day(self):
        name = "Saint Andrew's Day"
        self.assertNoHolidayName(name)

        obs_dts = (
            "2008-12-01",
            "2013-12-02",
            "2014-12-01",
            "2019-12-02",
        )
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "SCT":
                self.assertSubdivSctHolidayName(
                    name, (f"{year}-11-30" for year in range(2006, self.end_year))
                )
                self.assertNoSubdivSctHolidayName(name, range(self.start_year, 2006))
                self.assertSubdivSctHolidayName(f"{name} (observed)", obs_dts)
                self.assertNoSubdivSctNonObservedHoliday(obs_dts)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_christmas_day(self):
        name = "Christmas Day"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))

        obs_dts = (
            "2004-12-27",
            "2005-12-27",
            "2010-12-27",
            "2011-12-27",
            "2016-12-27",
            "2021-12-27",
            "2022-12-27",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_christmas_day_scotland(self):
        name = "Christmas Day"

        obs_dts = (
            "1955-12-26",
            "1960-12-26",
            "1966-12-26",
        )
        self.assertSubdivSctHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoSubdivSctNonObservedHoliday(obs_dts)
        self.assertNoHolidayName(f"{name} (observed)", obs_dts)

    def test_boxing_day(self):
        name = "Boxing Day"
        self.assertHolidayName(name, (f"{year}-12-26" for year in self.full_range))

        obs_dts = (
            "2004-12-28",
            "2009-12-28",
            "2010-12-28",
            "2015-12-28",
            "2020-12-28",
            "2021-12-28",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_boxing_day_scotland(self):
        name = "Boxing Day"
        self.assertSubdivSctHolidayName(
            name, (f"{year}-12-26" for year in range(1974, self.end_year))
        )
        self.assertNoSubdivSctHolidayName(name, range(self.start_year, 1974))

    def test_all_holidays_present(self):
        y_2015 = set()
        for subdiv in UnitedKingdom.subdivisions:
            y_2015.update(UnitedKingdom(observed=False, years=2015, subdiv=subdiv).values())

        all_holidays = {
            "New Year's Day",
            "New Year Holiday",
            "Saint Patrick's Day",
            "Good Friday",
            "Easter Monday",
            "May Day",
            "Spring Bank Holiday",
            "Summer Bank Holiday",
            "Battle of the Boyne",
            "Late Summer Bank Holiday",
            "Saint Andrew's Day",
            "Christmas Day",
            "Boxing Day",
        }
        self.assertEqual(all_holidays, y_2015)

    def test_l10n_default(self):
        # https://www.gov.uk/bank-holidays
        self.assertLocalizedHolidays(
            ("2024-01-01", "New Year's Day"),
            ("2024-01-02", "New Year Holiday"),
            ("2024-03-17", "Saint Patrick's Day"),
            ("2024-03-18", "Saint Patrick's Day (observed)"),
            ("2024-03-29", "Good Friday"),
            ("2024-04-01", "Easter Monday"),
            ("2024-05-06", "May Day"),
            ("2024-05-27", "Spring Bank Holiday"),
            ("2024-07-12", "Battle of the Boyne"),
            ("2024-08-05", "Summer Bank Holiday"),
            ("2024-08-26", "Late Summer Bank Holiday"),
            ("2024-11-30", "Saint Andrew's Day"),
            ("2024-12-02", "Saint Andrew's Day (observed)"),
            ("2024-12-25", "Christmas Day"),
            ("2024-12-26", "Boxing Day"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2024-01-01", "New Year's Day"),
            ("2024-01-02", "New Year Holiday"),
            ("2024-03-17", "Saint Patrick's Day"),
            ("2024-03-18", "Saint Patrick's Day (observed)"),
            ("2024-03-29", "Good Friday"),
            ("2024-04-01", "Easter Monday"),
            ("2024-05-06", "May Day"),
            ("2024-05-27", "Spring Bank Holiday"),
            ("2024-07-12", "Battle of the Boyne"),
            ("2024-08-05", "Summer Bank Holiday"),
            ("2024-08-26", "Late Summer Bank Holiday"),
            ("2024-11-30", "Saint Andrew's Day"),
            ("2024-12-02", "Saint Andrew's Day (observed)"),
            ("2024-12-25", "Christmas Day"),
            ("2024-12-26", "Boxing Day"),
        )

    def test_l10n_th(self):
        self.assertLocalizedHolidays(
            "th",
            ("2024-01-01", "วันขึ้นปีใหม่"),
            ("2024-01-02", "หยุดวันขึ้นปีใหม่"),
            ("2024-03-17", "วันนักบุญแพทริก"),
            ("2024-03-18", "ชดเชยวันนักบุญแพทริก"),
            ("2024-03-29", "วันศุกร์ประเสริฐ"),
            ("2024-04-01", "วันจันทร์อีสเตอร์"),
            ("2024-05-06", "วันเมย์เดย์"),
            ("2024-05-27", "วันหยุดฤดูใบไม้ผลิของธนาคาร"),
            ("2024-07-12", "วันรำลึกยุทธการแม่น้ำบอยน์"),
            ("2024-08-05", "วันหยุดฤดูร้อนของธนาคาร"),
            ("2024-08-26", "วันหยุดช่วงปลายฤดูร้อนของธนาคาร"),
            ("2024-11-30", "วันนักบุญแอนดรูว์"),
            ("2024-12-02", "ชดเชยวันนักบุญแอนดรูว์"),
            ("2024-12-25", "วันคริสต์มาส"),
            ("2024-12-26", "วันเปิดกล่องของขวัญ"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from datetime import date

from holidays.calendars.tibetan import _TibetanLunisolar
from holidays.groups.eastern import EasternCalendarHolidays


class TibetanCalendarHolidays(EasternCalendarHolidays):
    """
    Tibetan lunisolar calendar holidays.
    """

    def __init__(self, cls=None, *, show_estimated=False) -> None:
        self._tibetan_calendar = cls() if cls else _TibetanLunisolar()
        self._tibetan_calendar_show_estimated = show_estimated

    def _add_tibetan_calendar_holiday(
        self, name: str, dt_estimated: tuple[date | None, bool], days_delta: int = 0
    ) -> date | None:
        """
        Add Tibetan calendar holiday.

        Adds customizable estimation label to holiday name if holiday date
        is an estimation.
        """
        return self._add_eastern_calendar_holiday(
            name,
            dt_estimated,
            show_estimated=self._tibetan_calendar_show_estimated,
            days_delta=days_delta,
        )

    def _add_blessed_rainy_day(self, name) -> date | None:
        """
        Add Blessed Rainy Day (September Equinox of Autumn Calendar).
        """
        return self._add_tibetan_calendar_holiday(
            name, self._tibetan_calendar.blessed_rainy_day_date(self._year)
        )

    def _add_birth_of_guru_rinpoche(self, name) -> date | None:
        """
        Add Birth of Guru Rinpoche (10th day of the 5th lunar month).
        """
        return self._add_tibetan_calendar_holiday(
            name, self._tibetan_calendar.birth_of_guru_rinpoche_date(self._year)
        )

    def _add_buddha_first_sermon(self, name) -> date | None:
        """
        Add Buddha First Sermon (4th day of the 6th lunar month).
        """
        return self._add_tibetan_calendar_holiday(
            name, self._tibetan_calendar.buddha_first_sermon_date(self._year)
        )

    def _add_buddha_parinirvana(self, name) -> date | None:
        """
        Add Buddha Parinirvana (15th day of the 4th lunar month).
        """
        return self._add_tibetan_calendar_holiday(
            name, self._tibetan_calendar.buddha_parinirvana_date(self._year)
        )

    def _add_day_of_offering(self, name) -> date | None:
        """
        Add Day of Offering (1st day of the 12th lunar month).
        """
        return self._add_tibetan_calendar_holiday(
            name, self._tibetan_calendar.day_of_offering_date(self._year)
        )

    def _add_death_of_zhabdrung(self, name) -> date | None:
        """
        Add Death of Zhabdrung (10th day of the 3rd lunar month).
        """
        return self._add_tibetan_calendar_holiday(
            name, self._tibetan_calendar.death_of_zhabdrung_date(self._year)
        )

    def _add_descending_day_of_lord_buddha(self, name) -> date | None:
        """
        Add Descending Day of Lord Buddha (22nd day of the 9th lunar month).
        """
        return self._add_tibetan_calendar_holiday(
            name, self._tibetan_calendar.descending_day_of_lord_buddha_date(self._year)
        )

    def _add_losar(self, name) -> date | None:
        """
        Add Losar (1st day of the 1st lunar month).
        """
        return self._add_tibetan_calendar_holiday(
            name, self._tibetan_calendar.losar_date(self._year)
        )

    def _add_losar_day_two(self, name) -> date | None:
        """
        Add Losar Day Two.
        """
        return self._add_tibetan_calendar_holiday(
            name, self._tibetan_calendar.losar_date(self._year), days_delta=+1
        )

    def _add_thimphu_drubchen_day(self, name) -> date | None:
        """
        Add Thimphu Drubchen (6th day of the 8th lunar month).
        """
        return self._add_tibetan_calendar_holiday(
            name, self._tibetan_calendar.thimphu_drubchen_date(self._year), days_delta=+1
        )

    def _add_thimphu_tshechu_day(self, name) -> date | None:
        """
        Add Thimphu Tshechu (10th day of the 8th lunar month).
        """
        return self._add_tibetan_calendar_holiday(
            name, self._tibetan_calendar.thimphu_tshechu_date(self._year)
        )

    def _add_thimphu_tshechu_day_two(self, name) -> date | None:
        """
        Add Thimphu Tshechu Day 2.
        """
        return self._add_tibetan_calendar_holiday(
            name, self._tibetan_calendar.thimphu_tshechu_date(self._year), days_delta=+1
        )

    def _add_thimphu_tshechu_day_three(self, name) -> date | None:
        """
        Add Thimphu Tshechu Day 3.
        """
        return self._add_tibetan_calendar_holiday(
            name, self._tibetan_calendar.thimphu_tshechu_date(self._year), days_delta=+2
        )

    def _add_tibetan_winter_solstice(self, name) -> date | None:
        """
        Add Winter Solstice Day.
        """
        return self._add_tibetan_calendar_holiday(
            name, self._tibetan_calendar.tibetan_winter_solstice_date(self._year)
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.ghana import Ghana
from tests.common import CommonCountryTests


class TestGhana(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Ghana)

    def test_new_year_day(self):
        name = "New Year's Day"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        obs_dts = (
            "2012-01-02",
            "2017-01-02",
            "2022-01-03",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_constitution_day(self):
        name = "Constitution Day"
        self.assertHolidayName(name, (f"{year}-01-07" for year in range(2019, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2019))
        obs_dts = (
            "2023-01-09",
            "2024-01-08",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_independence_day(self):
        name = "Independence Day"
        self.assertHolidayName(name, (f"{year}-03-06" for year in self.full_range))
        obs_dts = (
            "2011-03-07",
            "2016-03-07",
            "2021-03-08",
            "2022-03-07",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Easter Monday"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_labor_day(self):
        name = "May Day"
        self.assertHolidayName(name, (f"{year}-05-01" for year in self.full_range))
        obs_dts = (
            "2011-05-02",
            "2016-05-02",
            "2021-05-03",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_founders_day(self):
        name = "Founders' Day"
        self.assertHolidayName(name, (f"{year}-08-04" for year in range(2019, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2019))
        obs_dts = (
            "2019-08-05",
            "2024-08-05",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_kwame_nkrumah_memorial_day(self):
        name_2009 = "Founder's Day"
        name_2019 = "Kwame Nkrumah Memorial Day"
        self.assertHolidayName(name_2009, (f"{year}-09-21" for year in range(2009, 2019)))
        self.assertHolidayName(name_2019, (f"{year}-09-21" for year in range(2019, self.end_year)))
        self.assertNoHolidayName(
            name_2009, range(self.start_year, 2009), range(2019, self.end_year)
        )
        self.assertNoHolidayName(name_2019, range(self.start_year, 2019))
        obs_dts_2009 = (
            "2013-09-23",
            "2014-09-22",
        )
        self.assertHolidayName(f"{name_2009} (observed)", obs_dts_2009)
        obs_dts_2019 = (
            "2019-09-23",
            "2024-09-23",
            "2025-09-22",
        )
        self.assertHolidayName(f"{name_2019} (observed)", obs_dts_2019)
        self.assertNoNonObservedHoliday(obs_dts_2009, obs_dts_2019)

    def test_farmers_day(self):
        name = "Farmer's Day"
        self.assertHolidayName(
            name,
            "2020-12-04",
            "2021-12-03",
            "2022-12-02",
            "2023-12-01",
            "2024-12-06",
            "2025-12-05",
        )
        self.assertHolidayName(name, self.full_range)

    def test_christmas_day(self):
        name = "Christmas Day"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))
        obs_dts = (
            "2011-12-27",
            "2016-12-27",
            "2021-12-27",
            "2022-12-27",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_boxing_day(self):
        name = "Boxing Day"
        self.assertHolidayName(name, (f"{year}-12-26" for year in self.full_range))
        obs_dts = (
            "2010-12-28",
            "2015-12-28",
            "2020-12-28",
            "2021-12-28",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_eid_al_fitr(self):
        name = "Eid ul-Fitr"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-05-24",
            "2021-05-13",
            "2022-05-02",
            "2023-04-21",
            "2024-04-10",
            "2025-03-30",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)
        obs_dts = (
            "2012-08-20",
            "2017-06-26",
            "2020-05-25",
            "2025-03-31",
        )
        self.assertIslamicNoEstimatedHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_eid_al_adha(self):
        name = "Eid ul-Adha"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-07-31",
            "2021-07-20",
            "2022-07-09",
            "2023-06-28",
            "2024-06-16",
            "2025-06-06",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)
        obs_dts = (
            "2016-09-12",
            "2019-08-12",
            "2022-07-11",
            "2024-06-17",
        )
        self.assertIslamicNoEstimatedHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_2023(self):
        self.assertHolidaysInYear(
            2023,
            ("2023-01-01", "New Year's Day"),
            ("2023-01-02", "New Year's Day (observed)"),
            ("2023-01-07", "Constitution Day"),
            ("2023-01-09", "Constitution Day (observed)"),
            ("2023-03-06", "Independence Day"),
            ("2023-04-07", "Good Friday"),
            ("2023-04-10", "Easter Monday"),
            ("2023-05-01", "May Day"),
            ("2023-04-21", "Eid ul-Fitr (estimated)"),
            ("2023-06-28", "Eid ul-Adha (estimated)"),
            ("2023-08-04", "Founders' Day"),
            ("2023-09-21", "Kwame Nkrumah Memorial Day"),
            ("2023-12-01", "Farmer's Day"),
            ("2023-12-25", "Christmas Day"),
            ("2023-12-26", "Boxing Day"),
        )

    def test_2024(self):
        self.assertHolidaysInYear(
            2024,
            ("2024-01-01", "New Year's Day"),
            ("2024-01-07", "Constitution Day"),
            ("2024-01-08", "Constitution Day (observed)"),
            ("2024-03-06", "Independence Day"),
            ("2024-03-29", "Good Friday"),
            ("2024-04-01", "Easter Monday"),
            ("2024-05-01", "May Day"),
            ("2024-04-10", "Eid ul-Fitr (estimated)"),
            ("2024-06-16", "Eid ul-Adha (estimated)"),
            ("2024-06-17", "Eid ul-Adha (observed, estimated)"),
            ("2024-08-04", "Founders' Day"),
            ("2024-08-05", "Founders' Day (observed)"),
            ("2024-09-21", "Kwame Nkrumah Memorial Day"),
            ("2024-09-23", "Kwame Nkrumah Memorial Day (observed)"),
            ("2024-12-06", "Farmer's Day"),
            ("2024-12-25", "Christmas Day"),
            ("2024-12-26", "Boxing Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

__all__ = ("DateLike", "HolidayBase", "HolidaySum")

import copy
import warnings
from bisect import bisect_left, bisect_right
from calendar import isleap
from collections.abc import Iterable
from datetime import date, datetime, timedelta, timezone
from functools import cached_property
from gettext import gettext, translation
from pathlib import Path
from typing import Any, Literal, Union, cast

from dateutil.parser import parse

from holidays.calendars.gregorian import (
    MON,
    TUE,
    WED,
    THU,
    FRI,
    SAT,
    SUN,
    _timedelta,
    _get_nth_weekday_from,
    _get_nth_weekday_of_month,
    DAYS,
    MONTHS,
    WEEKDAYS,
)
from holidays.constants import HOLIDAY_NAME_DELIMITER, PUBLIC, DEFAULT_START_YEAR, DEFAULT_END_YEAR
from holidays.helpers import _normalize_arguments, _normalize_tuple

CategoryArg = str | Iterable[str]
DateArg = date | tuple[int, int] | tuple[int, int, int]
DateLike = date | datetime | str | float | int
NameLookup = Literal["contains", "exact", "startswith", "icontains", "iexact", "istartswith"]
SpecialHoliday = tuple[int, int, str] | tuple[tuple[int, int, str], ...]
SubstitutedHoliday = (
    tuple[int, int, int, int]
    | tuple[int, int, int, int, int]
    | tuple[tuple[int, int, int, int] | tuple[int, int, int, int, int], ...]
)
YearArg = int | Iterable[int]


class HolidayBase(dict[date, str]):
    """Represent a dictionary-like collection of holidays for a specific country or region.

    This class inherits from `dict` and maps holiday dates to their names. It supports
    customization by country and, optionally, by province or state (subdivision). A date
    not present as a key is not considered a holiday (or, if `observed` is `False`, not
    considered an observed holiday).

    Keys are holiday dates, and values are corresponding holiday names. When accessing or
    assigning holidays by date, the following input formats are accepted:

    * `datetime.date`
    * `datetime.datetime`
    * `float` or `int` (Unix timestamp)
    * `str` of any format recognized by `dateutil.parser.parse()`

    Keys are always returned as `datetime.date` objects.

    To maximize performance, the holiday list is lazily populated one year at a time.
    On instantiation, the object is empty. Once a date is accessed, the full calendar
    year for that date is generated, unless `expand` is set to `False`. To pre-populate
    holidays, instantiate the class with the `years` argument:

        us_holidays = holidays.US(years=2020)

    It is recommended to use the
    [country_holidays()][holidays.utils.country_holidays] function for instantiation.

    Example usage:

        >>> from holidays import country_holidays
        >>> us_holidays = country_holidays('US')
        # For a specific subdivisions (e.g. state or province):
        >>> california_holidays = country_holidays('US', subdiv='CA')

    The below will cause 2015 holidays to be calculated on the fly:

        >>> from datetime import date
        >>> assert date(2015, 1, 1) in us_holidays

    This will be faster because 2015 holidays are already calculated:

        >>> assert date(2015, 1, 2) not in us_holidays

    The [`HolidayBase`][holidays.holiday_base.HolidayBase] class also recognizes strings
    of many formats and numbers representing a POSIX timestamp:

        >>> assert '2014-01-01' in us_holidays
        >>> assert '1/1/2014' in us_holidays
        >>> assert 1388597445 in us_holidays

    Show the holiday's name:

        >>> us_holidays.get('2014-01-01')
        "New Year's Day"

    Check a range:

        >>> us_holidays['2014-01-01': '2014-01-03']
        [datetime.date(2014, 1, 1)]

    List all 2020 holidays:

        >>> us_holidays = country_holidays('US', years=2020)
        >>> for day in sorted(us_holidays.items()):
        ...     print(day)
        (datetime.date(2020, 1, 1), "New Year's Day")
        (datetime.date(2020, 1, 20), 'Martin Luther King Jr. Day')
        (datetime.date(2020, 2, 17), "Washington's Birthday")
        (datetime.date(2020, 5, 25), 'Memorial Day')
        (datetime.date(2020, 7, 3), 'Independence Day (observed)')
        (datetime.date(2020, 7, 4), 'Independence Day')
        (datetime.date(2020, 9, 7), 'Labor Day')
        (datetime.date(2020, 10, 12), 'Columbus Day')
        (datetime.date(2020, 11, 11), 'Veterans Day')
        (datetime.date(2020, 11, 26), 'Thanksgiving Day')
        (datetime.date(2020, 12, 25), 'Christmas Day')

    Some holidays are only present in parts of a country:

        >>> us_pr_holidays = country_holidays('US', subdiv='PR')
        >>> assert '2018-01-06' not in us_holidays
        >>> assert '2018-01-06' in us_pr_holidays

    Append custom holiday dates by passing one of the following:

    * A dict mapping date values to holiday names (e.g. `{'2010-07-10': 'My birthday!'}`).
    * A list of date values (`datetime.date`, `datetime.datetime`, `str`, `int`, or `float`);
      each will be added with 'Holiday' as the default name.
    * A single date value of any of the supported types above; 'Holiday' will be used as
      the default name.

    ```python
    >>> custom_holidays = country_holidays('US', years=2015)
    >>> custom_holidays.update({'2015-01-01': "New Year's Day"})
    >>> custom_holidays.update(['2015-07-01', '07/04/2015'])
    >>> custom_holidays.update(date(2015, 12, 25))
    >>> assert date(2015, 1, 1) in custom_holidays
    >>> assert date(2015, 1, 2) not in custom_holidays
    >>> assert '12/25/2015' in custom_holidays
    ```

    For special (one-off) country-wide holidays handling use
    `special_public_holidays`:

        special_public_holidays = {
            1977: ((JUN, 7, "Silver Jubilee of Elizabeth II"),),
            1981: ((JUL, 29, "Wedding of Charles and Diana"),),
            1999: ((DEC, 31, "Millennium Celebrations"),),
            2002: ((JUN, 3, "Golden Jubilee of Elizabeth II"),),
            2011: ((APR, 29, "Wedding of William and Catherine"),),
            2012: ((JUN, 5, "Diamond Jubilee of Elizabeth II"),),
            2022: (
                (JUN, 3, "Platinum Jubilee of Elizabeth II"),
                (SEP, 19, "State Funeral of Queen Elizabeth II"),
            ),
        }

        def _populate(self, year):
            super()._populate(year)

            ...

    For more complex logic, like 4th Monday of January, you can inherit the
    [`HolidayBase`][holidays.holiday_base.HolidayBase] class and define your own `_populate()`
    method.
    See documentation for examples.
    """

    country: str
    """The country's ISO 3166-1 alpha-2 code."""
    market: str
    """The market's ISO 3166-1 alpha-2 code."""
    subdivisions: tuple[str, ...] = ()
    """The subdivisions supported for this country (see documentation)."""
    subdivisions_aliases: dict[str, str] = {}
    """Aliases for the ISO 3166-2 subdivision codes with the key as alias and
    the value the ISO 3166-2 subdivision code."""
    years: set[int]
    """The years calculated."""
    expand: bool
    """Whether the entire year is calculated when one date from that year
    is requested."""
    observed: bool
    """Whether dates when public holiday are observed are included."""
    subdiv: str | None = None
    """The subdiv requested as ISO 3166-2 code or one of the aliases."""
    special_holidays: dict[int, SpecialHoliday | SubstitutedHoliday] = {}
    """A list of the country-wide special (as opposite to regular) holidays for
    a specific year."""
    _deprecated_subdivisions: tuple[str, ...] = ()
    """Other subdivisions whose names are deprecated or aliases of the official
    ones."""
    weekend: set[int] = {SAT, SUN}
    """Country weekend days."""
    weekend_workdays: set[date]
    """Working days moved to weekends."""
    default_category: str = PUBLIC
    """The entity category used by default."""
    default_language: str | None = None
    """The entity language used by default."""
    categories: set[str] = set()
    """Requested holiday categories."""
    supported_categories: tuple[str, ...] = (PUBLIC,)
    """All holiday categories supported by this entity."""
    supported_languages: tuple[str, ...] = ()
    """All languages supported by this entity."""
    start_year: int = DEFAULT_START_YEAR
    """Start year of holidays presence for this entity."""
    end_year: int = DEFAULT_END_YEAR
    """End year of holidays presence for this entity."""
    parent_entity: type["HolidayBase"] | None = None
    """Optional parent entity to reference as a base."""

    def __init__(
        self,
        years: YearArg | None = None,
        expand: bool = True,
        observed: bool = True,
        subdiv: str | None = None,
        prov: str | None = None,  # Deprecated.
        state: str | None = None,  # Deprecated.
        language: str | None = None,
        categories: CategoryArg | None = None,
    ) -> None:
        """
        Args:
            years:
                The year(s) to pre-calculate public holidays for at instantiation.

            expand:
                Whether the entire year is calculated when one date from that year
                is requested.

            observed:
                Whether to include the dates when public holiday are observed
                (e.g. a holiday falling on a Sunday being observed the
                following Monday). This doesn't work for all countries.

            subdiv:
                The subdivision (e.g. state or province) as a ISO 3166-2 code
                or its alias; not implemented for all countries (see documentation).

            prov:
                *deprecated* use `subdiv` instead.

            state:
                *deprecated* use `subdiv` instead.

            language:
                Specifies the language in which holiday names are returned.

                Accepts either:

                * A two-letter ISO 639-1 language code (e.g., 'en' for English, 'fr' for French),
                    or
                * A language and entity combination using an underscore (e.g., 'en_US' for U.S.
                    English, 'pt_BR' for Brazilian Portuguese).

                !!! warning
                    The provided language or locale code must be supported by the holiday
                    entity. Unsupported values will result in names being shown in the entity's
                    original language.

                If not explicitly set (`language=None`), the system attempts to infer the
                language from the environment's locale settings. The following environment
                variables are checked, in order of precedence: LANGUAGE, LC_ALL, LC_MESSAGES, LANG.

                If none of these are set or they are empty, holiday names will default to the
                original language of the entity's holiday implementation.

                !!! warning
                    This fallback mechanism may yield inconsistent results across environments
                    (e.g., between a terminal session and a Jupyter notebook).

                To ensure consistent behavior, it is recommended to set the language parameter
                explicitly. If the specified language is not supported, holiday names will remain
                in the original language of the entity's holiday implementation.

                This behavior will be updated and formalized in v1.

            categories:
                Requested holiday categories.
        """
        super().__init__()

        # Categories validation.
        if self.default_category and self.default_category not in self.supported_categories:
            raise ValueError("The default category must be listed in supported categories.")

        if not self.default_category and not categories:
            raise ValueError("Categories cannot be empty if `default_category` is not set.")

        categories = _normalize_arguments(str, categories) or {self.default_category}
        if unknown_categories := categories.difference(  # type: ignore[union-attr]
            self.supported_categories
        ):
            raise ValueError(f"Category is not supported: {', '.join(unknown_categories)}.")

        # Subdivision validation.
        if subdiv := subdiv or prov or state:
            # Handle subdivisions passed as integers.
            if isinstance(subdiv, int):
                subdiv = str(subdiv)

            subdivision_aliases = tuple(self.subdivisions_aliases)
            supported_subdivisions = set(
                self.subdivisions
                + subdivision_aliases
                + self._deprecated_subdivisions
                + (self.parent_entity.subdivisions if self.parent_entity else ())
            )

            # Unsupported subdivisions.
            if not isinstance(self, HolidaySum) and subdiv not in supported_subdivisions:
                raise NotImplementedError(
                    f"Entity `{self._entity_code}` does not have subdivision {subdiv}"
                )

            # Deprecated arguments.
            if prov_state := prov or state:
                warnings.warn(
                    f"Arguments prov and state are deprecated, use subdiv='{prov_state}' instead.",
                    DeprecationWarning,
                )

            # Deprecated subdivisions.
            if subdiv in self._deprecated_subdivisions:
                warnings.warn(
                    "This subdivision is deprecated and will be removed after "
                    "Dec, 1 2023. The list of supported subdivisions: "
                    f"{', '.join(sorted(self.subdivisions))}; "
                    "the list of supported subdivisions aliases: "
                    f"{', '.join(sorted(subdivision_aliases))}.",
                    DeprecationWarning,
                )

        # Special holidays validation.
        if (has_substituted_holidays := getattr(self, "has_substituted_holidays", False)) and (
            not getattr(self, "substituted_label", None)
            or not getattr(self, "substituted_date_format", None)
        ):
            raise ValueError(
                f"Entity `{self._entity_code}` class must have `substituted_label` "
                "and `substituted_date_format` attributes set."
            )

        self.categories = categories
        self.expand = expand
        self.has_special_holidays = getattr(self, "has_special_holidays", False)
        self.has_substituted_holidays = has_substituted_holidays
        self.language = language
        self.observed = observed
        self.subdiv = subdiv
        self.weekend_workdays = getattr(self, "weekend_workdays", set())
        self.years = _normalize_arguments(int, years)

        # Configure l10n related attributes.
        self._init_translation()

        # Populate holidays.
        for year in self.years:
            self._populate(year)

    def __add__(
        self, other: Union[int, "HolidayBase", "HolidaySum"]
    ) -> "HolidayBase | HolidaySum":
        """Add another dictionary of public holidays creating a
        [`HolidaySum`][holidays.holiday_base.HolidaySum] object.

        Args:
            other:
                The dictionary of public holiday to be added.

        Returns:
            A [`HolidaySum`][holidays.holiday_base.HolidaySum]
            instance representing the combined holidays,
            or the original object if no combination occurs.

        Raises:
            TypeError:
                If `other` is not a `HolidayBase` or `HolidaySum`.
        """
        if isinstance(other, int) and other == 0:
            # Required to sum() list of holidays
            # sum([h1, h2]) is equivalent to (0 + h1 + h2).
            return self

        if not isinstance(other, (HolidayBase, HolidaySum)):
            raise TypeError("Holiday objects can only be added with other Holiday objects")

        return HolidaySum(self, other)

    def __bool__(self) -> bool:
        return len(self) > 0

    def __contains__(self, key: object) -> bool:
        """Check if a given date is a holiday.

        The method supports the following input types:

        * `datetime.date`
        * `datetime.datetime`
        * `float` or `int` (Unix timestamp)
        * `str` of any format recognized by `dateutil.parser.parse()`

        Args:
            key:
                The date to check.

        Returns:
            `True` if the date is a holiday, `False` otherwise.
        """

        if not isinstance(key, (date, datetime, float, int, str)):
            raise TypeError(f"Cannot convert type '{type(key)}' to date.")

        return dict.__contains__(cast("dict[Any, Any]", self), self.__keytransform__(key))

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, HolidayBase):
            return False

        for attribute_name in self.__attribute_names:
            if getattr(self, attribute_name, None) != getattr(other, attribute_name, None):
                return False

        return dict.__eq__(cast("dict[Any, Any]", self), other)

    def __getattr__(self, name):
        try:
            return self.__getattribute__(name)
        except AttributeError as e:
            # This part is responsible for _add_holiday_* syntactic sugar support.
            add_holiday_prefix = "_add_holiday_"
            # Raise early if prefix doesn't match to avoid patterns checks.
            if name[: len(add_holiday_prefix)] != add_holiday_prefix:
                raise e

            tokens = name.split("_")

            # Handle <month> <day> patterns (e.g., _add_holiday_jun_15()).
            if len(tokens) == 5:
                *_, month, day = tokens
                if month in MONTHS and day in DAYS:
                    return lambda name: self._add_holiday(
                        name, date(self._year, MONTHS[month], int(day))
                    )

            elif len(tokens) == 7:
                # Handle <last/nth> <weekday> of <month> patterns (e.g.,
                # _add_holiday_last_mon_of_aug() or _add_holiday_3rd_fri_of_aug()).
                *_, number, weekday, of, month = tokens
                if (
                    of == "of"
                    and (number == "last" or number[0].isdigit())
                    and month in MONTHS
                    and weekday in WEEKDAYS
                ):
                    return lambda name: self._add_holiday(
                        name,
                        _get_nth_weekday_of_month(
                            -1 if number == "last" else int(number[0]),
                            WEEKDAYS[weekday],
                            MONTHS[month],
                            self._year,
                        ),
                    )

                # Handle <n> days <past/prior> easter patterns (e.g.,
                # _add_holiday_8_days_past_easter() or
                # _add_holiday_5_days_prior_easter()).
                *_, days, unit, delta_direction, easter = tokens
                if (
                    unit in {"day", "days"}
                    and delta_direction in {"past", "prior"}
                    and easter == "easter"
                    and len(days) < 3
                    and days.isdigit()
                ):
                    return lambda name: self._add_holiday(
                        name,
                        _timedelta(
                            self._easter_sunday,
                            +int(days) if delta_direction == "past" else -int(days),
                        ),
                    )

            # Handle <n> day(s) <past/prior> <last/<nth> <weekday> of <month> patterns (e.g.,
            # _add_holiday_1_day_past_1st_fri_of_aug() or
            # _add_holiday_5_days_prior_last_fri_of_aug()).
            elif len(tokens) == 10:
                *_, days, unit, delta_direction, number, weekday, of, month = tokens
                if (
                    unit in {"day", "days"}
                    and delta_direction in {"past", "prior"}
                    and of == "of"
                    and len(days) < 3
                    and days.isdigit()
                    and (number == "last" or number[0].isdigit())
                    and month in MONTHS
                    and weekday in WEEKDAYS
                ):
                    return lambda name: self._add_holiday(
                        name,
                        _timedelta(
                            _get_nth_weekday_of_month(
                                -1 if number == "last" else int(number[0]),
                                WEEKDAYS[weekday],
                                MONTHS[month],
                                self._year,
                            ),
                            +int(days) if delta_direction == "past" else -int(days),
                        ),
                    )

            # Handle <nth> <weekday> <before/from> <month> <day> patterns (e.g.,
            # _add_holiday_1st_mon_before_jun_15() or _add_holiday_1st_mon_from_jun_15()).
            elif len(tokens) == 8:
                *_, number, weekday, date_direction, month, day = tokens
                if (
                    date_direction in {"before", "from"}
                    and number[0].isdigit()
                    and month in MONTHS
                    and weekday in WEEKDAYS
                    and day in DAYS
                ):
                    return lambda name: self._add_holiday(
                        name,
                        _get_nth_weekday_from(
                            -int(number[0]) if date_direction == "before" else +int(number[0]),
                            WEEKDAYS[weekday],
                            date(self._year, MONTHS[month], int(day)),
                        ),
                    )

            raise e  # No match.

    def __getitem__(self, key: DateLike) -> Any:
        if isinstance(key, slice):
            if not key.start or not key.stop:
                raise ValueError("Both start and stop must be given.")

            start = self.__keytransform__(key.start)
            stop = self.__keytransform__(key.stop)

            if key.step is None:
                step = 1
            elif isinstance(key.step, int):
                step = key.step
            elif isinstance(key.step, timedelta):
                step = key.step.days
            else:
                raise TypeError(f"Cannot convert type '{type(key.step)}' to int.")

            if step == 0:
                raise ValueError("Step value must not be zero.")

            diff_days = (stop - start).days
            if diff_days < 0 <= step or diff_days >= 0 > step:
                step = -step

            return [
                day
                for delta_days in range(0, diff_days, step)
                if (day := _timedelta(start, delta_days)) in self
            ]

        return dict.__getitem__(self, self.__keytransform__(key))

    def __getstate__(self) -> dict[str, Any]:
        """Return the object's state for serialization."""
        state = self.__dict__.copy()
        state.pop("tr", None)
        return state

    def __keytransform__(self, key: DateLike) -> date:
        """Convert various date-like formats to `datetime.date`.

        The method supports the following input types:

        * `datetime.date`
        * `datetime.datetime`
        * `float` or `int` (Unix timestamp)
        * `str` of any format recognized by `dateutil.parser.parse()`

        Args:
            key:
                The date-like object to convert.

        Returns:
            The corresponding `datetime.date` representation.
        """

        dt: date | None = None
        # Try to catch `date` and `str` type keys first.
        # Using type() here to skip date subclasses.
        # Key is `date`.
        if type(key) is date:
            dt = key

        # Key is `str` instance.
        elif isinstance(key, str):
            # key possibly contains a date in YYYY-MM-DD or YYYYMMDD format.
            if len(key) in {8, 10}:
                try:
                    dt = date.fromisoformat(key)
                except ValueError:
                    pass
            if dt is None:
                try:
                    dt = parse(key).date()
                except (OverflowError, ValueError):
                    raise ValueError(f"Cannot parse date from string '{key}'")

        # Key is `datetime` instance.
        elif isinstance(key, datetime):
            dt = key.date()

        # Must go after the `isinstance(key, datetime)` check as datetime is `date` subclass.
        elif isinstance(key, date):
            dt = key

        # Key is `float` or `int` instance.
        elif isinstance(key, (float, int)):
            dt = datetime.fromtimestamp(key, timezone.utc).date()

        # Key is not supported.
        else:
            raise TypeError(f"Cannot convert type '{type(key)}' to date.")

        # Automatically expand for `expand=True` cases.
        if self.expand and dt.year not in self.years:
            self.years.add(dt.year)
            self._populate(dt.year)

        return dt

    def __ne__(self, other: object) -> bool:
        if not isinstance(other, HolidayBase):
            return True

        for attribute_name in self.__attribute_names:
            if getattr(self, attribute_name, None) != getattr(other, attribute_name, None):
                return True

        return dict.__ne__(self, other)

    def __radd__(self, other: Any) -> "HolidayBase":
        return self.__add__(other)

    def __reduce__(self) -> str | tuple[Any, ...]:
        return super().__reduce__()

    def __repr__(self) -> str:
        if self:
            return super().__repr__()

        if hasattr(self, "market"):
            args = [repr(self.market)]

            if self.language is not None:
                args.append(f"language={self.language!r}")

            return f"holidays.financial_holidays({', '.join(args)})"

        elif hasattr(self, "country"):
            args = [repr(self.country)]

            if self.categories != {"public"}:
                args.append(f"categories={sorted(self.categories)!r}")

            if self.language is not None:
                args.append(f"language={self.language!r}")

            if self.subdiv:
                args.append(f"subdiv={self.subdiv!r}")

            return f"holidays.country_holidays({', '.join(args)})"

        return "holidays.HolidayBase()"

    def __setattr__(self, key: str, value: Any) -> None:
        dict.__setattr__(self, key, value)

        if self and key in {"categories", "observed"}:
            self.clear()
            for year in self.years:  # Re-populate holidays for each year.
                self._populate(year)

    def __setitem__(self, key: DateLike, value: str) -> None:
        if key in self:
            # If there are multiple holidays on the same date
            # order their names alphabetically.
            holiday_names = set(self[key].split(HOLIDAY_NAME_DELIMITER))
            holiday_names.update(value.split(HOLIDAY_NAME_DELIMITER))
            value = HOLIDAY_NAME_DELIMITER.join(sorted(holiday_names))

        dict.__setitem__(self, self.__keytransform__(key), value)

    def __setstate__(self, state: dict[str, Any]) -> None:
        """Restore the object's state after deserialization."""
        self.__dict__.update(state)
        self._init_translation()

    def __str__(self) -> str:
        if self:
            return super().__str__()

        parts = (
            f"'{attribute_name}': {getattr(self, attribute_name, None)}"
            for attribute_name in self.__attribute_names
        )

        return f"{{{', '.join(parts)}}}"

    @property
    def __attribute_names(self):
        return ("country", "expand", "language", "market", "observed", "subdiv", "years")

    @cached_property
    def _entity_code(self):
        return getattr(self, "country", None) or getattr(self, "market", None)

    @cached_property
    def _normalized_subdiv(self):
        return (
            self.subdivisions_aliases.get(self.subdiv, self.subdiv)
            .translate(
                str.maketrans(
                    {
                        "-": "_",
                        " ": "_",
                    }
                )
            )
            .lower()
        )

    @property
    def _sorted_categories(self):
        return (
            [self.default_category] + sorted(self.categories - {self.default_category})
            if self.default_category in self.categories
            else sorted(self.categories)
        )

    @classmethod
    def get_subdivision_aliases(cls) -> dict[str, list]:
        """Get subdivision aliases.

        Returns:
            A dictionary mapping subdivision aliases to their official
            [ISO 3166-2](https://en.wikipedia.org/wiki/ISO_3166-2) codes.
        """
        subdivision_aliases: dict[str, list[str]] = {s: [] for s in cls.subdivisions}
        for alias, subdivision in cls.subdivisions_aliases.items():
            subdivision_aliases[subdivision].append(alias)

        return subdivision_aliases

    def _init_translation(self) -> None:
        """Initialize translation function based on language settings."""
        supported_languages = set(self.supported_languages)
        if self._entity_code is not None:
            fallback = self.language not in supported_languages
            languages = [self.language] if self.language in supported_languages else None
            locale_directory = str(Path(__file__).with_name("locale"))

            # Add entity native content translations.
            entity_translation = translation(
                self._entity_code,
                fallback=fallback,
                languages=languages,
                localedir=locale_directory,
            )
            # Add a fallback if entity has parent translations.
            if parent_entity := self.parent_entity:
                entity_translation.add_fallback(
                    translation(
                        getattr(parent_entity, "country", None)
                        or getattr(parent_entity, "market", None),  # type: ignore[arg-type]
                        fallback=fallback,
                        languages=languages,
                        localedir=locale_directory,
                    )
                )
            self.tr = entity_translation.gettext
        else:
            self.tr = gettext

    def _is_leap_year(self) -> bool:
        """Returns True if the year is leap. Returns False otherwise."""
        return isleap(self._year)

    def _add_holiday(self, name: str, *args) -> date | None:
        """Add a holiday."""
        if not args:
            raise TypeError("Incorrect number of arguments.")

        dt = args if len(args) > 1 else args[0]
        dt = dt if isinstance(dt, date) else date(self._year, *dt)

        if dt.year != self._year:
            return None

        self[dt] = self.tr(name)
        return dt

    def _add_multiday_holiday(
        self, start_date: date, duration_days: int, *, name: str | None = None
    ) -> set[date]:
        """Add a multi-day holiday.

        Args:
            start_date:
                First day of the holiday.

            duration_days:
                Number of additional days to add.

            name:
                Optional holiday name; inferred from `start_date` if omitted.

        Returns:
            A set of all added holiday dates.

        Raises:
            ValueError:
                If the holiday name cannot be inferred from `start_date`.
        """
        if (holiday_name := name or self.get(start_date)) is None:
            raise ValueError(f"Cannot infer holiday name for date {start_date!r}.")

        return {
            d
            for delta in range(1, duration_days + 1)
            if (d := self._add_holiday(holiday_name, _timedelta(start_date, delta)))
        }

    def _add_special_holidays(self, mapping_names, *, observed=False):
        """Add special holidays."""
        for mapping_name in mapping_names:
            for data in _normalize_tuple(getattr(self, mapping_name, {}).get(self._year, ())):
                if len(data) == 3:  # Special holidays.
                    month, day, name = data
                    self._add_holiday(
                        self.tr(self.observed_label) % self.tr(name)
                        if observed
                        else self.tr(name),
                        month,
                        day,
                    )
                else:  # Substituted holidays.
                    to_month, to_day, from_month, from_day, *optional = data
                    from_date = date(optional[0] if optional else self._year, from_month, from_day)
                    self._add_holiday(
                        self.tr(self.substituted_label)
                        % from_date.strftime(self.tr(self.substituted_date_format)),
                        to_month,
                        to_day,
                    )
                    # when non-working day is transferred not from weekend, but from
                    # another transferred holiday (observed).
                    if self._is_weekend(from_date):
                        if from_date.year != self._year or from_date not in self:
                            self.weekend_workdays.add(from_date)
                    else:
                        if from_date.year == self._year and from_date in self:
                            self.pop(from_date)

    def _check_weekday(self, weekday: int, *args) -> bool:
        """
        Returns True if `weekday` equals to the date's week day.
        Returns False otherwise.
        """
        dt = args if len(args) > 1 else args[0]
        dt = dt if isinstance(dt, date) else date(self._year, *dt)
        return dt.weekday() == weekday

    def _get_weekend(self, dt: date) -> set[int]:
        return self.weekend

    def _is_monday(self, *args) -> bool:
        return self._check_weekday(MON, *args)

    def _is_tuesday(self, *args) -> bool:
        return self._check_weekday(TUE, *args)

    def _is_wednesday(self, *args) -> bool:
        return self._check_weekday(WED, *args)

    def _is_thursday(self, *args) -> bool:
        return self._check_weekday(THU, *args)

    def _is_friday(self, *args) -> bool:
        return self._check_weekday(FRI, *args)

    def _is_saturday(self, *args) -> bool:
        return self._check_weekday(SAT, *args)

    def _is_sunday(self, *args) -> bool:
        return self._check_weekday(SUN, *args)

    def _is_weekday(self, *args) -> bool:
        """
        Returns True if date's week day is not a weekend day.
        Returns False otherwise.
        """
        return not self._is_weekend(*args)

    def _is_weekend(self, *args) -> bool:
        """
        Returns True if date's week day is a weekend day.
        Returns False otherwise.
        """
        dt = args if len(args) > 1 else args[0]
        dt = dt if isinstance(dt, date) else date(self._year, *dt)
        return dt.weekday() in self._get_weekend(dt)

    def _populate(self, year: int) -> None:
        """This is a private method that populates (generates and adds) holidays
        for a given year. To keep things fast, it assumes that no holidays for
        the year have already been populated. It is required to be called
        internally by any country `populate()` method, while should not be called
        directly from outside.
        To add holidays to an object, use the [update()][holidays.holiday_base.HolidayBase.update]
        method.

        Args:
            year: The year to populate with holidays.

            >>> from holidays import country_holidays
            >>> us_holidays = country_holidays('US', years=2020)
            # to add new holidays to the object:
            >>> us_holidays.update(country_holidays('US', years=2021))
        """

        if year < self.start_year or year > self.end_year:
            return None

        self._year = year
        self._populate_common_holidays()
        self._populate_subdiv_holidays()

    def _populate_common_holidays(self):
        """Populate entity common holidays."""
        for category in self._sorted_categories:
            if pch_method := getattr(self, f"_populate_{category.lower()}_holidays", None):
                pch_method()

        if self.has_special_holidays:
            self._add_special_holidays(
                f"special_{category}_holidays" for category in self._sorted_categories
            )

    def _populate_subdiv_holidays(self):
        """Populate entity subdivision holidays."""
        if self.subdiv is None:
            return None

        for category in self._sorted_categories:
            if asch_method := getattr(
                self,
                f"_populate_subdiv_{self._normalized_subdiv}_{category.lower()}_holidays",
                None,
            ):
                asch_method()

        if self.has_special_holidays:
            self._add_special_holidays(
                f"special_{self._normalized_subdiv}_{category.lower()}_holidays"
                for category in self._sorted_categories
            )

    def append(self, *args: dict[DateLike, str] | list[DateLike] | DateLike) -> None:
        """Alias for [update()][holidays.holiday_base.HolidayBase.update] to mimic list type.

        Args:
            args:
                Holiday data to add. Can be:

                * A dictionary mapping dates to holiday names.
                * A list of dates (without names).
                * A single date.
        """
        return self.update(*args)

    def copy(self):
        """Return a copy of the object."""
        return copy.copy(self)

    def get(self, key: DateLike, default: str | Any = None) -> str | Any:
        """Retrieve the holiday name(s) for a given date.

        If the date is a holiday, returns the holiday name as a string.
        If multiple holidays fall on the same date, their names are joined by a semicolon (`;`).
        If the date is not a holiday, returns the provided `default` value (defaults to `None`).

        Args:
            key:
                The date expressed in one of the following types:

                * `datetime.date`
                * `datetime.datetime`
                * `float` or `int` (Unix timestamp)
                * `str` of any format recognized by `dateutil.parser.parse()`

            default:
                The default value to return if no value is found.

        Returns:
            The holiday name(s) as a string if the date is a holiday,
                or the `default` value otherwise.
        """
        return dict.get(self, self.__keytransform__(key), default)

    def get_list(self, key: DateLike) -> list[str]:
        """Retrieve all holiday names for a given date.

        Args:
            key:
                The date expressed in one of the following types:

                * `datetime.date`
                * `datetime.datetime`
                * `float` or `int` (Unix timestamp)
                * `str` of any format recognized by `dateutil.parser.parse()`

        Returns:
            A list of holiday names if the date is a holiday, otherwise an empty list.
        """
        return [name for name in self.get(key, "").split(HOLIDAY_NAME_DELIMITER) if name]

    def get_named(
        self,
        holiday_name: str,
        lookup: NameLookup = "icontains",
        split_multiple_names: bool = True,
    ) -> list[date]:
        """Find all holiday dates matching a given name.

        The search by default is case-insensitive and includes partial matches.

        Args:
            holiday_name:
                The holiday's name to try to match.

            lookup:
                The holiday name lookup type:

                * contains - case sensitive contains match;
                * exact - case sensitive exact match;
                * startswith - case sensitive starts with match;
                * icontains - case insensitive contains match;
                * iexact - case insensitive exact match;
                * istartswith - case insensitive starts with match;

            split_multiple_names:
                Either use the exact name for each date or split it by holiday
                name delimiter.

        Returns:
            A list of all holiday dates matching the provided holiday name.
        """
        holiday_name_dates = (
            ((k, name) for k, v in self.items() for name in v.split(HOLIDAY_NAME_DELIMITER))
            if split_multiple_names
            else ((k, v) for k, v in self.items())
        )

        if lookup == "icontains":
            holiday_name_lower = holiday_name.lower()
            return [dt for dt, name in holiday_name_dates if holiday_name_lower in name.lower()]
        elif lookup == "exact":
            return [dt for dt, name in holiday_name_dates if holiday_name == name]
        elif lookup == "contains":
            return [dt for dt, name in holiday_name_dates if holiday_name in name]
        elif lookup == "startswith":
            return [
                dt for dt, name in holiday_name_dates if holiday_name == name[: len(holiday_name)]
            ]
        elif lookup == "iexact":
            holiday_name_lower = holiday_name.lower()
            return [dt for dt, name in holiday_name_dates if holiday_name_lower == name.lower()]
        elif lookup == "istartswith":
            holiday_name_lower = holiday_name.lower()
            return [
                dt
                for dt, name in holiday_name_dates
                if holiday_name_lower == name[: len(holiday_name)].lower()
            ]

        raise AttributeError(f"Unknown lookup type: {lookup}")

    def get_closest_holiday(
        self,
        target_date: DateLike | None = None,
        direction: Literal["forward", "backward"] = "forward",
    ) -> tuple[date, str] | None:
        """Find the closest holiday relative to a given date.

        If `direction` is "forward", returns the next holiday after `target_date`.
        If `direction` is "backward", returns the previous holiday before `target_date`.
        If `target_date` is not provided, the current date is used.

        Args:
            target_date:
                The reference date. If None, defaults to today.

            direction:
                Search direction, either "forward" (next holiday) or
                "backward" (previous holiday).

        Returns:
            A tuple containing the holiday date and its name, or None if no holiday is found.
        """
        if direction not in {"backward", "forward"}:
            raise AttributeError(f"Unknown direction: {direction}")

        dt = self.__keytransform__(target_date or datetime.now().date())
        if direction == "forward" and (next_year := dt.year + 1) not in self.years:
            self._populate(next_year)
        elif direction == "backward" and (previous_year := dt.year - 1) not in self.years:
            self._populate(previous_year)

        sorted_dates = sorted(self.keys())
        position = (
            bisect_right(sorted_dates, dt)
            if direction == "forward"
            else bisect_left(sorted_dates, dt) - 1
        )
        if 0 <= position < len(sorted_dates):
            dt = sorted_dates[position]
            return dt, self[dt]

        return None

    def get_nth_working_day(self, key: DateLike, n: int) -> date:
        """Find the n-th working day from a given date.

        Moves forward if n is positive, or backward if n is negative.
        If n is 0, returns the given date if it is a working day; otherwise the next working day.

        Args:
            key:
                The starting date.

            n:
                The number of working days to move. Positive values move forward,
                negative values move backward.

        Returns:
            The calculated working day after shifting by n working days.
        """
        direction = +1 if n >= 0 else -1
        dt = self.__keytransform__(key)
        for _ in range(abs(n) or 1):
            if n:
                dt = _timedelta(dt, direction)
            while not self.is_working_day(dt):
                dt = _timedelta(dt, direction)
        return dt

    def get_working_days_count(self, start: DateLike, end: DateLike) -> int:
        """Calculate the number of working days between two dates.

        The date range works in a closed interval fashion [start, end] so both
        endpoints are included.

        Args:
            start:
                The range start date.

            end:
                The range end date.

        Returns:
            The total count of working days between the given dates.
        """
        dt1 = self.__keytransform__(start)
        dt2 = self.__keytransform__(end)
        if dt1 > dt2:
            dt1, dt2 = dt2, dt1
        days = (dt2 - dt1).days + 1
        return sum(self.is_working_day(_timedelta(dt1, n)) for n in range(days))

    def is_weekend(self, key: DateLike) -> bool:
        """Check if the given date's week day is a weekend day.

        Args:
            key:
                The date to check.

        Returns:
            True if the date's week day is a weekend day, False otherwise.
        """
        return self._is_weekend(self.__keytransform__(key))

    def is_working_day(self, key: DateLike) -> bool:
        """Check if the given date is considered a working day.

        Args:
            key:
                The date to check.

        Returns:
            True if the date is a working day, False if it is a holiday or weekend.
        """
        dt = self.__keytransform__(key)
        return dt in self.weekend_workdays if self._is_weekend(dt) else dt not in self

    def pop(self, key: DateLike, default: str | Any = None) -> str | Any:
        """Remove a holiday for a given date and return its name.

        If the specified date is a holiday, it will be removed, and its name will
        be returned. If the date is not a holiday, the provided `default` value
        will be returned instead.

        Args:
            key:
                The date expressed in one of the following types:

                * `datetime.date`
                * `datetime.datetime`
                * `float` or `int` (Unix timestamp)
                * `str` of any format recognized by `dateutil.parser.parse()`

            default:
                The default value to return if no match is found.

        Returns:
            The name of the removed holiday if the date was a holiday, otherwise
                the provided `default` value.

        Raises:
            KeyError: if date is not a holiday and default is not given.
        """
        if default is None:
            return dict.pop(self, self.__keytransform__(key))

        return dict.pop(self, self.__keytransform__(key), default)

    def pop_named(self, holiday_name: str, lookup: NameLookup = "icontains") -> list[date]:
        """Remove all holidays matching the given name.

        This method removes all dates associated with a holiday name, so they are
        no longer considered holidays. The search by default is case-insensitive and
        includes partial matches.

        Args:
            holiday_name:
                The holiday's name to try to match.

            lookup:
                The holiday name lookup type:

                * contains - case sensitive contains match;
                * exact - case sensitive exact match;
                * startswith - case sensitive starts with match;
                * icontains - case insensitive contains match;
                * iexact - case insensitive exact match;
                * istartswith - case insensitive starts with match;

        Returns:
            A list of dates removed.

        Raises:
            KeyError: if date is not a holiday.
        """
        use_exact_name = HOLIDAY_NAME_DELIMITER in holiday_name
        if not (
            dts := self.get_named(
                holiday_name, lookup=lookup, split_multiple_names=not use_exact_name
            )
        ):
            raise KeyError(holiday_name)

        popped = []
        for dt in dts:
            holiday_names = self[dt].split(HOLIDAY_NAME_DELIMITER)
            self.pop(dt)
            popped.append(dt)

            # Keep the rest of holidays falling on the same date.
            if use_exact_name:
                continue
            if lookup == "icontains":
                holiday_name_lower = holiday_name.lower()
                holiday_names = [
                    name for name in holiday_names if holiday_name_lower not in name.lower()
                ]
            elif lookup == "iexact":
                holiday_name_lower = holiday_name.lower()
                holiday_names = [
                    name for name in holiday_names if holiday_name_lower != name.lower()
                ]
            elif lookup == "istartswith":
                holiday_name_lower = holiday_name.lower()
                holiday_names = [
                    name
                    for name in holiday_names
                    if holiday_name_lower != name[: len(holiday_name)].lower()
                ]
            elif lookup == "contains":
                holiday_names = [name for name in holiday_names if holiday_name not in name]
            elif lookup == "exact":
                holiday_names = [name for name in holiday_names if holiday_name != name]
            else:  # startswith
                holiday_names = [
                    name for name in holiday_names if holiday_name != name[: len(holiday_name)]
                ]
            if holiday_names:
                self[dt] = HOLIDAY_NAME_DELIMITER.join(holiday_names)

        return popped

    def update(  # type: ignore[override]
        self, *args: dict[DateLike, str] | list[DateLike] | DateLike
    ) -> None:
        """Update the object, overwriting existing dates.

        Args:
            args:
                Either another dictionary object where keys are dates and values
                are holiday names, or a single date (or a list of dates) for which
                the value will be set to "Holiday".

                Dates can be expressed in one or more of the following types:

                * `datetime.date`
                * `datetime.datetime`
                * `float` or `int` (Unix timestamp)
                * `str` of any format recognized by `dateutil.parser.parse()`
        """
        for arg in args:
            if isinstance(arg, dict):
                for key, value in arg.items():
                    self[key] = value
            elif isinstance(arg, list):
                for item in arg:
                    self[item] = "Holiday"
            else:
                self[arg] = "Holiday"


class HolidaySum(HolidayBase):
    """
    Combine multiple holiday collections into a single dictionary-like object.

    This class represents the sum of two or more [`HolidayBase`][holidays.holiday_base.HolidayBase]
    instances. The resulting object behaves like a dictionary mapping dates to holiday
    names, with the following behaviors:

    * The `holidays` attribute stores the original holiday collections as a list.
    * The `country` and `subdiv` attributes are combined from all operands and
      may become lists.
    * If multiple holidays fall on the same date, their names are merged.
    * Holidays are generated (expanded) for all years included in the operands.
    """

    country: str | list[str]  # type: ignore[assignment]
    """Countries included in the addition."""
    market: str | list[str]  # type: ignore[assignment]
    """Markets included in the addition."""
    subdiv: str | list[str] | None  # type: ignore[assignment]
    """Subdivisions included in the addition."""
    holidays: list[HolidayBase]
    """The original HolidayBase objects included in the addition."""
    years: set[int]
    """The years calculated."""

    def __init__(
        self, h1: Union[HolidayBase, "HolidaySum"], h2: Union[HolidayBase, "HolidaySum"]
    ) -> None:
        """
        Args:
            h1:
                The first HolidayBase object to add.

            h2:
                The other HolidayBase object to add.

        Example:

            >>> from holidays import country_holidays
            >>> nafta_holidays = country_holidays('US', years=2020) + \
    country_holidays('CA') + country_holidays('MX')
            >>> dates = sorted(nafta_holidays.items(), key=lambda x: x[0])
            >>> from pprint import pprint
            >>> pprint(dates[:10], width=72)
            [(datetime.date(2020, 1, 1), "Año Nuevo; New Year's Day"),
             (datetime.date(2020, 1, 20), 'Martin Luther King Jr. Day'),
             (datetime.date(2020, 2, 3), 'Día de la Constitución'),
             (datetime.date(2020, 2, 17), "Washington's Birthday"),
             (datetime.date(2020, 3, 16), 'Natalicio de Benito Juárez'),
             (datetime.date(2020, 4, 10), 'Good Friday'),
             (datetime.date(2020, 5, 1), 'Día del Trabajo'),
             (datetime.date(2020, 5, 25), 'Memorial Day'),
             (datetime.date(2020, 7, 1), 'Canada Day'),
             (datetime.date(2020, 7, 3), 'Independence Day (observed)')]
        """
        # Store originals in the holidays attribute.
        self.holidays = []
        for operand in (h1, h2):
            if isinstance(operand, HolidaySum):
                self.holidays.extend(operand.holidays)
            else:
                self.holidays.append(operand)

        # Join years, expand and observed.
        kwargs: dict[str, Any] = {
            "expand": h1.expand or h2.expand,
            "observed": h1.observed or h2.observed,
            "years": h1.years | h2.years,
        }
        # Join country and subdivisions data.
        # TODO: this way makes no sense: joining Italy Catania (IT, CA) with
        # USA Mississippi (US, MS) and USA Michigan (US, MI) yields
        # country=["IT", "US"] and subdiv=["CA", "MS", "MI"], which could very
        # well be California and Messina and Milano, or Catania, Mississippi
        # and Milano, or ... you get the picture.
        # Same goes when countries and markets are being mixed (working, yet
        # still nonsensical).
        value: str | list[str] | None
        for attr in ("country", "market", "subdiv"):
            a1 = getattr(h1, attr, None)
            a2 = getattr(h2, attr, None)
            if a1 and a2 and a1 != a2:
                a1 = a1 if isinstance(a1, list) else [a1]
                a2 = a2 if isinstance(a2, list) else [a2]
                value = a1 + a2
            else:
                value = a1 or a2

            if attr == "subdiv":
                kwargs[attr] = value
            else:
                setattr(self, attr, value)

        # Retain language if they match and are strings.
        # If language wasn't assigned, default_language acts as fallback.
        h1_language = h1.language or h1.default_language
        h2_language = h2.language or h2.default_language
        if isinstance(h1_language, str) and h1_language == h2_language:
            kwargs["language"] = h1_language

        HolidayBase.__init__(self, **kwargs)

        # supported_languages is used for iCalExporter language check as well.
        self.supported_languages = (h1_language,) if h1_language else ()

    def _populate(self, year):
        for operand in self.holidays:
            operand._populate(year)
            self.update(cast("dict[DateLike, str]", operand))



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.constants import GOVERNMENT, HALF_DAY
from holidays.countries.puerto_rico import PuertoRico
from tests.common import CommonCountryTests


class TestPuertoRico(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(PuertoRico)

    def test_no_holidays(self):
        super().test_no_holidays()

        self.assertNoHolidays(
            PuertoRico(categories=GOVERNMENT, years=range(self.start_year, 2001))
        )
        self.assertNoHolidays(PuertoRico(categories=HALF_DAY, years=range(self.start_year, 2003)))

    def test_epiphany(self):
        name = "Epiphany"
        self.assertHolidayName(name, (f"{year}-01-06" for year in range(1932, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1932))

    def test_birthday_of_eugenio_maria_de_hostos(self):
        name = "Birthday of Eugenio María de Hostos"
        self.assertHolidayName(name, (f"{year}-01-11" for year in range(1940, 1971)))
        self.assertHolidayName(
            name,
            "2010-01-11",
            "2011-01-10",
            "2012-01-09",
            "2013-01-14",
            "2014-01-13",
        )
        self.assertHolidayName(name, range(1971, 2015))
        self.assertNoHolidayName(name, range(self.start_year, 1940), range(2015, self.end_year))
        obs_dts = (
            "1953-01-12",
            "1959-01-12",
            "1970-01-12",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_the_day_of_the_women_and_men_heroes_of_puerto_rico(self):
        name_1903 = "George Washington Day"
        name_2015 = "George Washington Day, Presidents' Day, and the Puerto Rican Heroes Day"
        name_2016 = (
            "George Washington Day, Presidents' Day, and the Day of the Hero "
            "and the Illustrious Woman of Puerto Rico"
        )
        name_2019 = (
            "George Washington Day, Presidents' Day, and the Day of the Women and Men "
            "Heroes of Puerto Rico"
        )
        self.assertHolidayName(name_1903, (f"{year}-02-22" for year in range(1903, 1971)))
        self.assertHolidayName(
            name_1903,
            "2010-02-15",
            "2011-02-21",
            "2012-02-20",
            "2013-02-18",
            "2014-02-17",
        )
        self.assertHolidayName(name_1903, range(1971, 2015))
        self.assertHolidayName(name_2015, "2015-02-16")
        self.assertHolidayName(
            name_2016,
            "2016-02-15",
            "2017-02-20",
            "2018-02-19",
        )
        self.assertHolidayName(
            name_2019,
            "2019-02-18",
            "2020-02-17",
            "2021-02-15",
            "2022-02-21",
            "2023-02-20",
            "2024-02-19",
            "2025-02-17",
        )
        self.assertHolidayName(name_2019, range(2019, self.end_year))
        self.assertNoHolidayName(name_1903, range(2019, self.end_year))
        self.assertNoHolidayName(name_2015, range(1903, 2015), range(2016, self.end_year))
        self.assertNoHolidayName(name_2016, range(1903, 2016), range(2019, self.end_year))
        self.assertNoHolidayName(name_2019, range(1903, 2019))
        obs_dts = (
            "1953-02-23",
            "1959-02-23",
            "1970-02-23",
        )
        self.assertHolidayName(f"{name_1903} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_emancipation_day(self):
        name = "Emancipation Day"
        self.assertHolidayName(name, (f"{year}-03-22" for year in self.full_range))
        obs_dts = (
            "1998-03-23",
            "2009-03-23",
            "2015-03-23",
            "2020-03-23",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_jose_de_diego_day(self):
        name = "José de Diego Day"
        self.assertHolidayName(name, (f"{year}-04-16" for year in range(1925, 1971)))
        self.assertHolidayName(
            name,
            "2010-04-19",
            "2011-04-18",
            "2012-04-16",
            "2013-04-15",
            "2014-04-21",
        )
        self.assertHolidayName(name, range(1971, 2015))
        self.assertNoHolidayName(name, range(self.start_year, 1925), range(2015, self.end_year))
        obs_dts = (
            "1961-04-17",
            "1967-04-17",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_luis_munoz_rivera_day(self):
        name = "Luis Muñoz Rivera Day"
        self.assertHolidayName(name, (f"{year}-07-17" for year in range(1918, 1971)))
        self.assertHolidayName(
            name,
            "2010-07-19",
            "2011-07-18",
            "2012-07-16",
            "2013-07-15",
            "2014-07-21",
        )
        self.assertHolidayName(name, range(1971, 2015))
        self.assertNoHolidayName(name, range(self.start_year, 1918), range(2015, self.end_year))
        obs_dts = (
            "1955-07-18",
            "1960-07-18",
            "1966-07-18",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_constitution_day(self):
        name_1903 = "Occupation Day"
        name_1953 = "Puerto Rico Constitution Day"
        self.assertHolidayName(name_1903, (f"{year}-07-25" for year in range(1903, 1953)))
        self.assertHolidayName(name_1953, (f"{year}-07-25" for year in range(1953, self.end_year)))
        self.assertNoHolidayName(name_1903, range(1953, self.end_year))
        self.assertNoHolidayName(name_1953, range(self.start_year, 1953))
        obs_dts = (
            "2004-07-26",
            "2010-07-26",
            "2021-07-26",
        )
        self.assertHolidayName(f"{name_1953} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_jose_celso_barbosa_day(self):
        name = "José Celso Barbosa Day"
        self.assertHolidayName(
            name, (f"{year}-07-27" for year in (*range(1938, 1971), *range(1994, 2015)))
        )
        self.assertHolidayName(
            name,
            "1989-07-24",
            "1990-07-23",
            "1991-07-22",
            "1992-07-27",
            "1993-07-26",
        )
        self.assertHolidayName(name, range(1971, 1994))
        self.assertNoHolidayName(name, range(self.start_year, 1938), range(2015, self.end_year))
        obs_dts = (
            "1958-07-28",
            "2003-07-28",
            "2008-07-28",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_puerto_rican_identity_day(self):
        name_1938 = "Discovery of Puerto Rico Day"
        name_2014 = "Puerto Rican Culture and Discovery of Puerto Rico Day"
        name_2023 = "Puerto Rican Identity Day"
        self.assertHolidayName(name_1938, (f"{year}-11-19" for year in range(1938, 2014)))
        self.assertHolidayName(name_2014, (f"{year}-11-19" for year in range(2014, 2023)))
        self.assertHolidayName(name_2023, (f"{year}-11-19" for year in range(2023, self.end_year)))
        self.assertNoHolidayName(
            name_1938, range(self.start_year, 1938), range(2014, self.end_year)
        )
        self.assertNoHolidayName(
            name_2014, range(self.start_year, 2014), range(2023, self.end_year)
        )
        self.assertNoHolidayName(name_2023, range(self.start_year, 2023))

        obs_dts_1938 = (
            "2000-11-20",
            "2006-11-20",
        )
        obs_dts_2014 = ("2017-11-20",)
        obs_dts_2023 = ("2023-11-20",)
        self.assertHolidayName(f"{name_1938} (observed)", obs_dts_1938)
        self.assertHolidayName(f"{name_2014} (observed)", obs_dts_2014)
        self.assertHolidayName(f"{name_2023} (observed)", obs_dts_2023)
        self.assertNoNonObservedHoliday(obs_dts_1938, obs_dts_2014, obs_dts_2023)

    def test_american_citizenship_day(self):
        name = "American Citizenship Day"
        self.assertNoHolidayName(name)
        self.assertGovernmentHolidayName(
            name, (f"{year}-03-02" for year in range(2001, self.end_year))
        )
        self.assertNoGovernmentHolidayName(name, range(self.start_year, 2001))

    def test_christmas_eve(self):
        name = "Christmas Eve (from 12pm)"
        self.assertHalfDayHolidayName(
            name, (f"{year}-12-24" for year in range(2003, self.end_year))
        )
        self.assertNoHalfDayHolidayName(name, range(self.start_year, 2003))

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "New Year's Day"),
            ("2022-01-06", "Epiphany"),
            ("2022-01-17", "Martin Luther King Jr. Day"),
            ("2022-02-14", "Valentine's Day"),
            (
                "2022-02-21",
                "George Washington Day, Presidents' Day, and the Day of the Women and Men "
                "Heroes of Puerto Rico",
            ),
            ("2022-03-02", "American Citizenship Day"),
            ("2022-03-17", "Saint Patrick's Day"),
            ("2022-03-22", "Emancipation Day"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-17", "Easter Sunday"),
            ("2022-05-08", "Mother's Day"),
            ("2022-05-30", "Memorial Day"),
            ("2022-06-19", "Father's Day; Juneteenth National Independence Day"),
            ("2022-06-20", "Juneteenth National Independence Day (observed)"),
            ("2022-07-04", "Independence Day"),
            ("2022-07-25", "Puerto Rico Constitution Day"),
            ("2022-09-05", "Labor Day"),
            ("2022-10-10", "Columbus Day"),
            ("2022-10-31", "Halloween"),
            ("2022-11-11", "Veterans Day"),
            ("2022-11-19", "Puerto Rican Culture and Discovery of Puerto Rico Day"),
            ("2022-11-24", "Thanksgiving Day"),
            ("2022-12-24", "Christmas Eve; Christmas Eve (from 12pm)"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Christmas Day (observed)"),
            ("2022-12-31", "New Year's Eve"),
        )

    def test_l10n_th(self):
        self.assertLocalizedHolidays(
            "th",
            ("2022-01-01", "วันขึ้นปีใหม่"),
            ("2022-01-06", "วันสมโภชพระคริสต์แสดงองค์"),
            ("2022-01-17", "วันมาร์ติน ลูเทอร์ คิง จูเนียร์"),
            ("2022-02-14", "วันวาเลนไทน์"),
            ("2022-02-21", "วันจอร์จ วอชิงตัน, วันประธานาธิบดี และวันวีรบุรุษและวีรสตรีแห่งเปอร์โตริโก"),
            ("2022-03-02", "วันแห่งความเป็นพลเมืองอเมริกัน"),
            ("2022-03-17", "วันนักบุญแพทริก"),
            ("2022-03-22", "วันเลิกทาส"),
            ("2022-04-15", "วันศุกร์ประเสริฐ"),
            ("2022-04-17", "วันอาทิตย์อีสเตอร์"),
            ("2022-05-08", "วันแม่"),
            ("2022-05-30", "วันรำลึก"),
            ("2022-06-19", "วันประกาศอิสรภาพแห่งชาติจูนทีนท์; วันพ่อ"),
            ("2022-06-20", "ชดเชยวันประกาศอิสรภาพแห่งชาติจูนทีนท์"),
            ("2022-07-04", "วันประกาศอิสรภาพ"),
            ("2022-07-25", "วันรัฐธรรมนูญเปอร์โตริโก"),
            ("2022-09-05", "วันแรงงาน"),
            ("2022-10-10", "วันโคลัมบัส"),
            ("2022-10-31", "วันฮาโลวีน"),
            ("2022-11-11", "วันทหารผ่านศึก"),
            ("2022-11-19", "วันวัฒนธรรมเปอร์โตริโกและการค้นพบเปอร์โตริโก"),
            ("2022-11-24", "วันขอบคุณพระเจ้า"),
            ("2022-12-24", "วันคริสต์มาสอีฟ; วันคริสต์มาสอีฟ (ตั้งแต่ 12:00 น.)"),
            ("2022-12-25", "วันคริสต์มาส"),
            ("2022-12-26", "ชดเชยวันคริสต์มาส"),
            ("2022-12-31", "วันสิ้นปี"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.united_arab_emirates import UnitedArabEmirates
from tests.common import CommonCountryTests


class TestUnitedArabEmirates(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(UnitedArabEmirates)

    def test_special(self):
        self.assertHoliday(
            # 29 Ramadan Eid al-Fitr specials.
            "2020-05-22",
            "2021-05-11",
            "2022-04-30",
            "2024-04-08",
            # Other Special Public Holidays.
            "2022-05-14",
            "2022-05-15",
            "2022-05-16",
        )

    def test_special_government(self):
        self.assertGovernmentHoliday(
            # Extended holidays for Public Sectors.
            "2022-05-05",
            "2022-05-06",
            "2022-05-07",
            "2022-05-08",
            "2023-12-04",
        )

    def test_special_optional(self):
        # 2019 Pope Visit.
        self.assertOptionalHoliday("2019-02-05")

    def test_commemoration_day(self):
        name = "يوم الشهيد"
        self.assertHolidayName(name, (f"{year}-11-30" for year in range(2015, 2019)))
        self.assertHolidayName(name, (f"{year}-12-01" for year in range(2019, 2024)))
        self.assertNoHolidayName(name, range(self.start_year, 2015), range(2024, self.end_year))

    def test_eid_al_fitr(self):
        name = "عيد الفطر"
        name_holiday = "عطلة عيد الفطر"
        self.assertHolidayName(
            name,
            "2017-06-25",
            "2018-06-15",
            "2019-06-04",
            "2020-05-24",
            "2021-05-13",
            "2022-05-02",
            "2023-04-21",
            "2024-04-10",
            "2025-03-30",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)
        self.assertIslamicNoEstimatedHolidayName(name_holiday, self.full_range)
        self.assertNoHolidayName(name_holiday, "2018-06-14", "2025-03-29")

    def test_eid_al_adha(self):
        name = "عيد الأضحى"
        self.assertHolidayName(
            name,
            "2017-09-01",
            "2018-08-22",
            "2019-08-11",
            "2020-07-31",
            "2021-07-20",
            "2022-07-09",
            "2023-06-28",
            "2024-06-16",
            "2025-06-04",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)
        self.assertIslamicNoEstimatedHolidayName("عطلة عيد الأضحى", self.full_range)
        self.assertIslamicNoEstimatedHolidayName("وقفة عرفة", self.full_range)

    def test_islamic_new_year(self):
        name = "رأس السنة الهجرية"
        self.assertHolidayName(
            name,
            "2017-09-22",
            "2018-09-11",
            "2019-08-31",
            "2020-08-23",
            "2021-08-12",
            "2022-07-30",
            "2023-07-21",
            "2024-07-07",
            "2025-06-26",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_prophets_birthday(self):
        name = "عيد المولد النبوي"
        self.assertHolidayName(
            name,
            "2017-11-30",
            "2018-11-18",
            "2019-11-09",
            "2020-10-29",
            "2021-10-21",
            "2022-10-08",
            "2023-09-29",
            "2024-09-15",
            "2025-09-05",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_isra_and_miraj(self):
        name = "ليلة المعراج"
        self.assertHolidayName(name, "2017-04-23", "2018-04-14")
        self.assertIslamicNoEstimatedHolidayName(name, range(self.start_year, 2019))
        self.assertNoIslamicNoEstimatedHolidayName(name, range(2019, self.end_year))

    def test_weekend(self):
        for dt in (
            "2021-12-24",  # FRI.
            "2021-12-25",  # SAT.
            "2021-12-31",  # FRI.
            "2022-01-01",  # SAT.
            "2022-01-02",  # SUN.
        ):
            self.assertTrue(self.holidays.is_weekend(dt))

        for dt in (
            "2021-12-26",  # SUN.
            "2022-01-07",  # FRI.
        ):
            self.assertFalse(self.holidays.is_weekend(dt))

    def test_2020(self):
        # https://gulfbusiness.com/revealed-uae-private-sector-holidays-for-eid-al-fitr-2020/
        # https://www.timeanddate.com/holidays/united-arab-emirates/2020?hol=134217729
        self.assertHolidaysInYear(
            2020,
            ("2020-01-01", "رأس السنة الميلادية"),
            ("2020-05-22", "عطلة عيد الفطر"),
            ("2020-05-23", "عطلة عيد الفطر"),
            ("2020-05-24", "عيد الفطر"),
            ("2020-05-25", "عطلة عيد الفطر"),
            ("2020-05-26", "عطلة عيد الفطر"),
            ("2020-07-30", "وقفة عرفة"),
            ("2020-07-31", "عيد الأضحى"),
            ("2020-08-01", "عطلة عيد الأضحى"),
            ("2020-08-02", "عطلة عيد الأضحى"),
            ("2020-08-23", "رأس السنة الهجرية"),
            ("2020-10-29", "عيد المولد النبوي"),
            ("2020-12-01", "يوم الشهيد"),
            ("2020-12-02", "اليوم الوطني"),
            ("2020-12-03", "اليوم الوطني"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2018-01-01", "رأس السنة الميلادية"),
            ("2018-04-14", "ليلة المعراج"),
            ("2018-06-15", "عيد الفطر"),
            ("2018-06-16", "عطلة عيد الفطر"),
            ("2018-06-17", "عطلة عيد الفطر"),
            ("2018-08-21", "وقفة عرفة"),
            ("2018-08-22", "عيد الأضحى"),
            ("2018-08-23", "عطلة عيد الأضحى"),
            ("2018-08-24", "عطلة عيد الأضحى"),
            ("2018-09-11", "رأس السنة الهجرية"),
            ("2018-11-18", "عيد المولد النبوي"),
            ("2018-11-30", "يوم الشهيد"),
            ("2018-12-02", "اليوم الوطني"),
            ("2018-12-03", "اليوم الوطني"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2018-01-01", "New Year's Day"),
            ("2018-04-14", "Isra' and Mi'raj"),
            ("2018-06-15", "Eid al-Fitr"),
            ("2018-06-16", "Eid al-Fitr Holiday"),
            ("2018-06-17", "Eid al-Fitr Holiday"),
            ("2018-08-21", "Arafat Day"),
            ("2018-08-22", "Eid al-Adha"),
            ("2018-08-23", "Eid al-Adha Holiday"),
            ("2018-08-24", "Eid al-Adha Holiday"),
            ("2018-09-11", "Islamic New Year"),
            ("2018-11-18", "Prophet's Birthday"),
            ("2018-11-30", "Commemoration Day"),
            ("2018-12-02", "National Day"),
            ("2018-12-03", "National Day"),
        )

    def test_l10n_th(self):
        self.assertLocalizedHolidays(
            "th",
            ("2018-01-01", "วันขึ้นปีใหม่"),
            ("2018-04-14", "วันเมี๊ยะราจ"),
            ("2018-06-15", "วันอีฎิ้ลฟิตริ"),
            ("2018-06-16", "เทศกาลอีฎิ้ลฟิตริ"),
            ("2018-06-17", "เทศกาลอีฎิ้ลฟิตริ"),
            ("2018-08-21", "วันอารอฟะห์"),
            ("2018-08-22", "วันอีดิ้ลอัฎฮา"),
            ("2018-08-23", "เทศกาลอีดิ้ลอัฎฮา"),
            ("2018-08-24", "เทศกาลอีดิ้ลอัฎฮา"),
            ("2018-09-11", "วันขึ้นปีใหม่อิสลาม"),
            ("2018-11-18", "วันเมาลิดนบี"),
            ("2018-11-30", "วันรำลึกผู้อุทิศตน"),
            ("2018-12-02", "วันชาติสหรัฐอาหรับเอมิเรตส์"),
            ("2018-12-03", "วันชาติสหรัฐอาหรับเอมิเรตส์"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.albania import Albania
from tests.common import CommonCountryTests


class TestAlbania(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Albania)

    def test_special_holidays(self):
        self.assertHolidayName(
            "Ditë pushimi",
            "2020-01-03",
            "2022-03-21",
            "2024-03-15",
        )

    def test_new_years_day(self):
        name = "Festat e Vitit të Ri"
        self.assertHolidayName(
            name,
            (f"{year}-01-01" for year in self.full_range),
            (f"{year}-01-02" for year in self.full_range),
        )
        obs_dts = (
            "2012-01-03",
            "2016-01-04",
            "2017-01-03",
            "2021-01-04",
            "2022-01-03",
            "2022-01-04",
            "2023-01-03",
        )
        self.assertHolidayName(f"{name} (ditë pushimi e shtyrë)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_summer_day(self):
        name = "Dita e Verës"
        self.assertHolidayName(name, (f"{year}-03-14" for year in range(2004, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2004))
        obs_dts = (
            "2010-03-15",
            "2015-03-16",
            "2020-03-16",
            "2021-03-15",
        )
        self.assertHolidayName(f"{name} (ditë pushimi e shtyrë)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_nowruz_day(self):
        name = "Dita e Nevruzit"
        self.assertHolidayName(name, (f"{year}-03-22" for year in range(1996, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1996))
        obs_dts = (
            "2014-03-24",
            "2015-03-23",
            "2020-03-23",
        )
        self.assertHolidayName(f"{name} (ditë pushimi e shtyrë)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_catholic_easter_sunday(self):
        name = "E diela e Pashkëve Katolike"
        self.assertHolidayName(
            name,
            "2018-04-01",
            "2019-04-21",
            "2020-04-12",
            "2021-04-04",
            "2022-04-17",
            "2023-04-09",
            "2024-03-31",
        )
        self.assertHolidayName(name, self.full_range)
        obs_dts = (
            # Special.
            "2008-03-25",
        )
        self.assertHolidayName(f"{name} (ditë pushimi e shtyrë)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_orthodox_easter_sunday(self):
        name = "E diela e Pashkëve Ortodokse"
        self.assertHolidayName(
            name,
            "2018-04-08",
            "2019-04-28",
            "2020-04-19",
            "2021-05-02",
            "2022-04-24",
            "2023-04-16",
            "2024-05-05",
        )
        self.assertHolidayName(name, self.full_range)
        obs_dts = (
            # Special.
            "2000-05-02",
            "2021-05-04",
            "2027-05-04",
        )
        self.assertHolidayName(f"{name} (ditë pushimi e shtyrë)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_international_workers_day(self):
        name = "Dita Ndërkombëtare e Punëtorëve"
        self.assertHolidayName(name, (f"{year}-05-01" for year in self.full_range))
        obs_dts = (
            "2010-05-03",
            "2011-05-02",
            "2016-05-02",
            "2021-05-03",
        )
        self.assertHolidayName(f"{name} (ditë pushimi e shtyrë)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_mother_teresa_day(self):
        name_blessed = "Dita e Lumturimit të Shenjt Terezës"
        name_saint = "Dita e Shenjtërimit të Shenjt Terezës"
        self.assertHolidayName(name_blessed, (f"{year}-10-19" for year in range(2004, 2018)))
        self.assertNoHolidayName(
            name_blessed, range(self.start_year, 2004), range(2018, self.end_year)
        )
        self.assertHolidayName(
            name_saint, (f"{year}-09-05" for year in range(2018, self.end_year))
        )
        self.assertNoHolidayName(name_saint, range(self.start_year, 2018))
        obs_dts_blessed = (
            "2013-10-21",
            "2014-10-20",
        )
        self.assertHolidayName(f"{name_blessed} (ditë pushimi e shtyrë)", obs_dts_blessed)
        obs_dts_saint = (
            "2020-09-07",
            "2021-09-06",
        )
        self.assertHolidayName(f"{name_saint} (ditë pushimi e shtyrë)", obs_dts_saint)
        self.assertNoNonObservedHoliday(obs_dts_blessed, obs_dts_saint)

    def test_alphabet_day(self):
        name = "Dita e Alfabetit"
        self.assertHolidayName(name, (f"{year}-11-22" for year in range(2024, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2024))
        obs_dts = (
            "2025-11-24",
            "2026-11-23",
        )
        self.assertHolidayName(f"{name} (ditë pushimi e shtyrë)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_flag_and_independence_day(self):
        name = "Dita Flamurit dhe e Pavarësisë"
        self.assertHolidayName(name, (f"{year}-11-28" for year in self.full_range))
        obs_dts = (
            "2010-11-30",
            "2015-11-30",
            "2020-11-30",
            "2021-11-30",
        )
        self.assertHolidayName(f"{name} (ditë pushimi e shtyrë)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_liberation_day(self):
        name = "Dita e Çlirimit"
        self.assertHolidayName(name, (f"{year}-11-29" for year in self.full_range))
        obs_dts = (
            "2014-12-01",
            "2015-12-01",
            "2020-12-01",
        )
        self.assertHolidayName(f"{name} (ditë pushimi e shtyrë)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_national_youth_day(self):
        name = "Dita Kombëtare e Rinisë"
        self.assertHolidayName(name, (f"{year}-12-08" for year in range(2009, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2009))
        obs_dts = (
            "2012-12-10",
            "2013-12-09",
            "2018-12-10",
            "2019-12-09",
            "2024-12-09",
        )
        self.assertHolidayName(f"{name} (ditë pushimi e shtyrë)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_christmas_day(self):
        name = "Krishtlindjet"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))
        obs_dts = (
            "2010-12-27",
            "2011-12-26",
            "2016-12-26",
            "2021-12-27",
            "2022-12-26",
        )
        self.assertHolidayName(f"{name} (ditë pushimi e shtyrë)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_eid_al_fitr(self):
        name = "Dita e Bajramit të Madh"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2018-06-15",
            "2019-06-04",
            "2020-05-24",
            "2021-05-13",
            "2022-05-02",
            "2023-04-21",
            "2024-04-10",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)
        obs_dts = (
            "2012-08-20",
            "2017-06-26",
            "2020-05-25",
        )
        self.assertIslamicNoEstimatedHolidayName(f"{name} (ditë pushimi e shtyrë)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_eid_al_adha(self):
        name = "Dita e Kurban Bajramit"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2006-01-10",
            "2006-12-31",
            "2018-08-21",
            "2019-08-11",
            "2020-07-31",
            "2021-07-20",
            "2022-07-09",
            "2023-06-28",
            "2024-06-16",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)
        obs_dts = (
            "2007-01-03",
            "2014-10-06",
            "2016-09-12",
            "2019-08-12",
            "2022-07-11",
            "2024-06-17",
        )
        self.assertIslamicNoEstimatedHolidayName(f"{name} (ditë pushimi e shtyrë)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_2022(self):
        self.assertHolidaysInYear(
            2022,
            ("2022-01-01", "Festat e Vitit të Ri"),
            ("2022-01-02", "Festat e Vitit të Ri"),
            ("2022-01-03", "Festat e Vitit të Ri (ditë pushimi e shtyrë)"),
            ("2022-01-04", "Festat e Vitit të Ri (ditë pushimi e shtyrë)"),
            ("2022-03-14", "Dita e Verës"),
            ("2022-03-21", "Ditë pushimi"),
            ("2022-03-22", "Dita e Nevruzit"),
            ("2022-04-17", "E diela e Pashkëve Katolike"),
            ("2022-04-18", "E diela e Pashkëve Katolike (ditë pushimi e shtyrë)"),
            ("2022-04-24", "E diela e Pashkëve Ortodokse"),
            ("2022-04-25", "E diela e Pashkëve Ortodokse (ditë pushimi e shtyrë)"),
            ("2022-05-01", "Dita Ndërkombëtare e Punëtorëve"),
            ("2022-05-02", "Dita e Bajramit të Madh"),
            ("2022-05-03", "Dita Ndërkombëtare e Punëtorëve (ditë pushimi e shtyrë)"),
            ("2022-07-09", "Dita e Kurban Bajramit"),
            ("2022-07-11", "Dita e Kurban Bajramit (ditë pushimi e shtyrë)"),
            ("2022-09-05", "Dita e Shenjtërimit të Shenjt Terezës"),
            ("2022-11-28", "Dita Flamurit dhe e Pavarësisë"),
            ("2022-11-29", "Dita e Çlirimit"),
            ("2022-12-08", "Dita Kombëtare e Rinisë"),
            ("2022-12-25", "Krishtlindjet"),
            ("2022-12-26", "Krishtlindjet (ditë pushimi e shtyrë)"),
        )

    def test_2023(self):
        self.assertHolidaysInYear(
            2023,
            ("2023-01-01", "Festat e Vitit të Ri"),
            ("2023-01-02", "Festat e Vitit të Ri"),
            ("2023-01-03", "Festat e Vitit të Ri (ditë pushimi e shtyrë)"),
            ("2023-03-14", "Dita e Verës"),
            ("2023-03-22", "Dita e Nevruzit"),
            ("2023-04-09", "E diela e Pashkëve Katolike"),
            ("2023-04-10", "E diela e Pashkëve Katolike (ditë pushimi e shtyrë)"),
            ("2023-04-16", "E diela e Pashkëve Ortodokse"),
            ("2023-04-17", "E diela e Pashkëve Ortodokse (ditë pushimi e shtyrë)"),
            ("2023-04-21", "Dita e Bajramit të Madh"),
            ("2023-05-01", "Dita Ndërkombëtare e Punëtorëve"),
            ("2023-06-28", "Dita e Kurban Bajramit"),
            ("2023-09-05", "Dita e Shenjtërimit të Shenjt Terezës"),
            ("2023-11-28", "Dita Flamurit dhe e Pavarësisë"),
            ("2023-11-29", "Dita e Çlirimit"),
            ("2023-12-08", "Dita Kombëtare e Rinisë"),
            ("2023-12-25", "Krishtlindjet"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2024-01-01", "Festat e Vitit të Ri"),
            ("2024-01-02", "Festat e Vitit të Ri"),
            ("2024-03-14", "Dita e Verës"),
            ("2024-03-15", "Ditë pushimi"),
            ("2024-03-22", "Dita e Nevruzit"),
            ("2024-03-31", "E diela e Pashkëve Katolike"),
            ("2024-04-01", "E diela e Pashkëve Katolike (ditë pushimi e shtyrë)"),
            ("2024-04-10", "Dita e Bajramit të Madh"),
            ("2024-05-01", "Dita Ndërkombëtare e Punëtorëve"),
            ("2024-05-05", "E diela e Pashkëve Ortodokse"),
            ("2024-05-06", "E diela e Pashkëve Ortodokse (ditë pushimi e shtyrë)"),
            ("2024-06-16", "Dita e Kurban Bajramit"),
            ("2024-06-17", "Dita e Kurban Bajramit (ditë pushimi e shtyrë)"),
            ("2024-09-05", "Dita e Shenjtërimit të Shenjt Terezës"),
            ("2024-11-22", "Dita e Alfabetit"),
            ("2024-11-28", "Dita Flamurit dhe e Pavarësisë"),
            ("2024-11-29", "Dita e Çlirimit"),
            ("2024-12-08", "Dita Kombëtare e Rinisë"),
            ("2024-12-09", "Dita Kombëtare e Rinisë (ditë pushimi e shtyrë)"),
            ("2024-12-25", "Krishtlindjet"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2024-01-01", "New Year's Day"),
            ("2024-01-02", "New Year's Day"),
            ("2024-03-14", "Summer Day"),
            ("2024-03-15", "Public Holiday"),
            ("2024-03-22", "Nowruz Day"),
            ("2024-03-31", "Catholic Easter Sunday"),
            ("2024-04-01", "Catholic Easter Sunday (observed)"),
            ("2024-04-10", "Eid al-Fitr"),
            ("2024-05-01", "International Workers' Day"),
            ("2024-05-05", "Orthodox Easter Sunday"),
            ("2024-05-06", "Orthodox Easter Sunday (observed)"),
            ("2024-06-16", "Eid al-Adha"),
            ("2024-06-17", "Eid al-Adha (observed)"),
            ("2024-09-05", "Mother Teresa Canonization Day"),
            ("2024-11-22", "Alphabet Day"),
            ("2024-11-28", "Flag and Independence Day"),
            ("2024-11-29", "Liberation Day"),
            ("2024-12-08", "National Youth Day"),
            ("2024-12-09", "National Youth Day (observed)"),
            ("2024-12-25", "Christmas Day"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2024-01-01", "Новий рік"),
            ("2024-01-02", "Новий рік"),
            ("2024-03-14", "День літа"),
            ("2024-03-15", "Вихідний день"),
            ("2024-03-22", "Свято Новруз"),
            ("2024-03-31", "Великдень (католицький)"),
            ("2024-04-01", "Великдень (католицький) (вихідний)"),
            ("2024-04-10", "Рамазан-байрам"),
            ("2024-05-01", "Міжнародний день трудящих"),
            ("2024-05-05", "Великдень (православний)"),
            ("2024-05-06", "Великдень (православний) (вихідний)"),
            ("2024-06-16", "Курбан-байрам"),
            ("2024-06-17", "Курбан-байрам (вихідний)"),
            ("2024-09-05", "День канонізації матері Терези"),
            ("2024-11-22", "День алфавіту"),
            ("2024-11-28", "День прапора та незалежності"),
            ("2024-11-29", "День визволення"),
            ("2024-12-08", "Національний день молоді"),
            ("2024-12-09", "Національний день молоді (вихідний)"),
            ("2024-12-25", "Різдво Христове"),
        )
